@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 
 class rms ~kind ~window_length ~update source =
 object (self)
-  inherit Source.operator kind [source]
+  inherit Source.operator ~name:"rms" kind [source]
 
   method stype = source#stype
   method is_ready = source#is_ready
