@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -29,10 +29,9 @@ let get_clock = Tutils.lazy_cell (fun () -> new Clock.self_sync "pulse")
 let error_translator e =
    match e with
      | Pulseaudio.Error n ->
-       raise (Utils.Translation
-         (Printf.sprintf
-            "Pulseaudio error: %s" (Pulseaudio.string_of_error n)))
-     | _ -> ()
+        Some (Printf.sprintf
+                "Pulseaudio error: %s" (Pulseaudio.string_of_error n))
+     | _ -> None
 
 let () = Utils.register_error_translator error_translator
 
