@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -96,11 +96,13 @@ let mk_wav params =
 let mp3_base_defaults = 
     { Encoder.MP3.
         stereo = true ;
+        stereo_mode = Encoder.MP3.Joint_stereo ;
         samplerate = 44100 ;
         bitrate_control = Encoder.MP3.CBR 128 ;
+        internal_quality = 2;
         id3v2 = None ;
         msg_interval = 0.1 ;
-        msg = Configure.vendor }
+        msg = "" }
 
 let mp3_base f =
   function
@@ -108,6 +110,19 @@ let mp3_base f =
         { f with Encoder.MP3.stereo = b }
     | ("mono",{ term = Bool b }) ->
         { f with Encoder.MP3.stereo = not b }
+    | ("stereo_mode",({ term = String m } as t)) ->
+        let mode =
+          match m with
+            | "default" -> Encoder.MP3.Default
+            | "joint_stereo" -> Encoder.MP3.Joint_stereo
+            | "stereo" -> Encoder.MP3.Stereo
+            | _ -> raise (Error(t,"Invalid stereo mode!"))
+        in
+        { f with Encoder.MP3.stereo_mode = mode }
+    | ("internal_quality",({ term = Int q } as t)) ->
+        if q < 0 || q > 9 then
+          raise (Error(t,"Internal quality must be a value between 0 and 9!"));
+        { f with Encoder.MP3.internal_quality = q }
     | ("msg_interval",{ term = Float i }) ->
         { f with Encoder.MP3.msg_interval = i }
     | ("msg",{ term = String m }) ->
@@ -245,6 +260,32 @@ let mk_mp3_vbr params =
   in
     mk (Encoder (Encoder.MP3 mp3))
 
+let mk_shine params =
+  let defaults =
+    { Encoder.Shine.
+        channels = 2 ;
+        samplerate = 44100 ;
+        bitrate = 128 }
+  in
+  let shine =
+    List.fold_left
+      (fun f ->
+        function
+          | ("channels",{ term = Int i }) ->
+              { f with Encoder.Shine.channels = i }
+          | ("samplerate",{ term = Int i }) ->
+              { f with Encoder.Shine.samplerate = i }
+          | ("bitrate",{ term = Int i }) ->
+              { f with Encoder.Shine.bitrate = i }
+          | ("",{ term = Var s }) when String.lowercase s = "mono" ->
+              { f with Encoder.Shine.channels = 1 }
+          | ("",{ term = Var s }) when String.lowercase s = "stereo" ->
+              { f with Encoder.Shine.channels = 2 }
+          | (_,t) -> raise (generic_error t))
+      defaults params
+  in
+    mk (Encoder (Encoder.Shine shine))
+
 let mk_aacplus params =
   let defaults =
     { Encoder.AACPlus.
@@ -300,10 +341,72 @@ let mk_voaacenc params =
   in
     mk (Encoder (Encoder.VoAacEnc voaacenc))
 
+let mk_fdkaac params =
+  let defaults =
+    { Encoder.FdkAacEnc.
+        afterburner    = false;
+        aot            = `Mpeg_2 `HE_AAC_v2;
+        bitrate        = 64;
+        channels       = 2;
+        samplerate     = 44100;
+        sbr_mode       = false;
+        transmux       = `Adts }
+  in
+  let valid_samplerates = [
+    8000;  11025; 12000; 16000; 22050; 24000; 32000;
+    44100; 48000; 64000; 88200; 96000 ]
+  in
+  let fdkaac =
+    List.fold_left
+      (fun f ->
+        function
+          | ("afterburner",{ term = Bool b }) ->
+              { f with Encoder.FdkAacEnc.afterburner = b }
+          | ("aot",({ term = String s } as t)) ->
+              let aot = try Encoder.FdkAacEnc.aot_of_string s with
+                | Not_found -> raise (Error (t,"invalid aot value"))
+              in
+              { f with Encoder.FdkAacEnc.aot = aot }
+          | ("bitrate",{ term = Int i }) ->
+              { f with Encoder.FdkAacEnc.bitrate = i }
+          | ("channels",{ term = Int i }) ->
+              { f with Encoder.FdkAacEnc.channels = i }
+          | ("samplerate",({ term = Int i } as t)) ->
+              if not (List.mem i valid_samplerates) then
+               begin
+                let err =
+                  Printf.sprintf "invalid samplerate value. Possible values: %s"
+                  (String.concat ", "
+                    (List.map string_of_int valid_samplerates))
+                in
+                raise (Error (t,err));
+               end;
+              { f with Encoder.FdkAacEnc.samplerate = i }
+          | ("sbr_mode",{ term = Bool b }) ->
+              { f with Encoder.FdkAacEnc.sbr_mode = b }
+          | ("transmux",({ term = String s } as t)) ->
+              let transmux = try Encoder.FdkAacEnc.transmux_of_string s with
+                | Not_found -> raise (Error (t,"invalid transmux value"))
+              in
+              { f with Encoder.FdkAacEnc.transmux = transmux }
+          | ("",{ term = Var s }) when String.lowercase s = "mono" ->
+              { f with Encoder.FdkAacEnc.channels = 1 }
+          | ("",{ term = Var s }) when String.lowercase s = "stereo" ->
+              { f with Encoder.FdkAacEnc.channels = 2 }
+          | (_,t) -> raise (generic_error t))
+      defaults params
+  in
+  let aot = fdkaac.Encoder.FdkAacEnc.aot in
+    if aot = `Mpeg_4 `HE_AAC_v2 || aot = `Mpeg_2 `HE_AAC_v2 then
+      if fdkaac.Encoder.FdkAacEnc.channels <> 2 then
+        failwith "HE-AAC v2 is only available with 2 channels.";
+    mk (Encoder (Encoder.FdkAacEnc fdkaac))
+
 let mk_flac_gen params =
   let defaults =
     { Encoder.Flac.
         channels = 2 ;
+        fill = None;
         samplerate = 44100 ;
         bits_per_sample = 16;
         compression = 5 }
@@ -323,6 +426,8 @@ let mk_flac_gen params =
               if i <> 8 && i <> 16 && i <> 32 then
                 raise (Error (t,"invalid bits_per_sample value")) ;
               { f with Encoder.Flac.bits_per_sample = i }
+          | ("bytes_per_page",{ term = Int i }) ->
+              { f with Encoder.Flac.fill = Some i }
           | ("",{ term = Var s }) when String.lowercase s = "mono" ->
               { f with Encoder.Flac.channels = 1 }
           | ("",{ term = Var s }) when String.lowercase s = "stereo" ->
@@ -374,11 +479,103 @@ let mk_external params =
       raise Encoder.External.No_process ;
     mk (Encoder (Encoder.External ext))
 
+let mk_opus params =
+  let defaults =
+    { Encoder.Opus.
+        application = None ;
+        complexity = None ;
+        max_bandwidth = None ;
+        mode = Encoder.Opus.VBR true ;
+        bitrate = `Auto ;
+        fill = None;
+        channels = 2 ;
+        samplerate = 48000 ;
+        signal = None ;
+        frame_size = 20.;
+        dtx = false;
+    }
+  in
+  let opus =
+    List.fold_left
+      (fun f ->
+        function
+          | ("application",{ term = String "voip" }) ->
+              { f with Encoder.Opus.application = Some `Voip }
+          | ("application",{ term = String "audio" }) ->
+              { f with Encoder.Opus.application = Some `Audio }
+          | ("application",{ term = String "restricted_lowdelay" }) ->
+              { f with Encoder.Opus.application = Some `Restricted_lowdelay }
+          | ("complexity",({ term = Int c } as t)) ->
+              (* Doc say this should be from 0 to 10. *)
+              if c < 0 || c > 10 then
+                raise (Error (t,"Opus complexity should be in 0..10"));
+              { f with Encoder.Opus.complexity = Some c }
+          | ("max_bandwidth",{ term = String "narrow_band" }) ->
+              { f with Encoder.Opus.max_bandwidth = Some `Narrow_band }
+          | ("max_bandwidth",{ term = String "medium_band" }) ->
+              { f with Encoder.Opus.max_bandwidth = Some `Medium_band }
+          | ("max_bandwidth",{ term = String "wide_band" }) ->
+              { f with Encoder.Opus.max_bandwidth = Some `Wide_band }
+          | ("max_bandwidth",{ term = String "super_wide_band" }) ->
+              { f with Encoder.Opus.max_bandwidth = Some `Super_wide_band }
+          | ("max_bandwidth",{ term = String "full_band" }) ->
+              { f with Encoder.Opus.max_bandwidth = Some `Full_band }
+          | ( "frame_size", ({ term = Float size } as t)) ->
+              let frame_sizes = [2.5;5.;10.;20.;40.;60.] in
+              if not (List.mem size frame_sizes) then
+                raise (Error (t,"Opus frame size should be one of \
+                                 2.5, 5., 10., 20., 40. or 60."));
+              { f with Encoder.Opus.frame_size = size }
+          | ("samplerate",({ term = Int i } as t)) ->
+              let samplerates = [8000;12000;16000;24000;48000] in
+              if not (List.mem i samplerates) then
+                raise (Error (t,"Opus samplerate should be one of \
+                                 8000, 12000, 16000, 24000 or 48000"));
+              { f with Encoder.Opus.samplerate = i }
+          | ("bitrate",({ term = Int i } as t)) ->
+              let i = i*1000 in
+              (* Doc say this should be from 500 to 512000. *)
+              if i < 500 || i > 512000 then
+                raise (Error (t,"Opus bitrate should be in 5..512"));
+              { f with Encoder.Opus.bitrate = `Bitrate i }
+          | ("bitrate",{ term = String "auto" }) ->
+              { f with Encoder.Opus.bitrate = `Auto }
+          | ("bitrate",{ term = String "max" }) ->
+              { f with Encoder.Opus.bitrate = `Bitrate_max }
+          | ("channels",({ term = Int i } as t)) ->
+              if i < 1 or i > 2 then
+                raise (Error (t,"Only mono and stereo streams are supported \
+                                 for now."));
+              { f with Encoder.Opus.channels = i }
+          | ("vbr",{ term = String "none" }) ->
+              { f with Encoder.Opus.mode = Encoder.Opus.CBR }
+          | ("vbr",{ term = String "constrained" }) ->
+              { f with Encoder.Opus.mode = Encoder.Opus.VBR true }
+          | ("vbr",{ term = String "unconstrained" }) ->
+              { f with Encoder.Opus.mode = Encoder.Opus.VBR false }
+          | ("signal",{ term = String "voice" }) ->
+              { f with Encoder.Opus.signal = Some `Voice }
+          | ("signal",{ term = String "music" }) ->
+              { f with Encoder.Opus.signal = Some `Music }
+          | ("bytes_per_page",{ term = Int i }) ->
+              { f with Encoder.Opus.fill = Some i }
+          | ("dtx",{ term = Bool b }) ->
+              { f with Encoder.Opus.dtx = b }
+          | ("",{ term = Var s }) when String.lowercase s = "mono" ->
+              { f with Encoder.Opus.channels = 1 }
+          | ("",{ term = Var s }) when String.lowercase s = "stereo" ->
+              { f with Encoder.Opus.channels = 2 }
+          | (_,t) -> raise (generic_error t))
+      defaults params
+  in
+    Encoder.Ogg.Opus opus
+
 let mk_vorbis_cbr params =
   let defaults =
     { Encoder.Vorbis.
         mode = Encoder.Vorbis.CBR 128 ;
         channels = 2 ;
+        fill = None;
         samplerate = 44100 ;
     }
   in
@@ -392,6 +589,8 @@ let mk_vorbis_cbr params =
               { f with Encoder.Vorbis.mode = Encoder.Vorbis.CBR i }
           | ("channels",{ term = Int i }) ->
               { f with Encoder.Vorbis.channels = i }
+          | ("bytes_per_page",{ term = Int i }) ->
+              { f with Encoder.Vorbis.fill = Some i }
           | ("",{ term = Var s }) when String.lowercase s = "mono" ->
               { f with Encoder.Vorbis.channels = 1 }
           | ("",{ term = Var s }) when String.lowercase s = "stereo" ->
@@ -406,6 +605,7 @@ let mk_vorbis_abr params =
     { Encoder.Vorbis.
         mode = Encoder.Vorbis.ABR (None,None,None) ;
         channels = 2 ;
+        fill = None ;
         samplerate = 44100 ;
     }
   in
@@ -431,6 +631,8 @@ let mk_vorbis_abr params =
               { f with Encoder.Vorbis.mode = Encoder.Vorbis.ABR (Some i,x,y) }
           | ("channels",{ term = Int i }) ->
               { f with Encoder.Vorbis.channels = i }
+          | ("bytes_per_page",{ term = Int i }) ->
+              { f with Encoder.Vorbis.fill = Some i }
           | ("",{ term = Var s }) when String.lowercase s = "mono" ->
               { f with Encoder.Vorbis.channels = 1 }
           | ("",{ term = Var s }) when String.lowercase s = "stereo" ->
@@ -445,6 +647,7 @@ let mk_vorbis params =
     { Encoder.Vorbis.
         mode = Encoder.Vorbis.VBR 0.3 ;
         channels = 2 ;
+        fill = None ;
         samplerate = 44100 ;
     }
   in
@@ -465,6 +668,8 @@ let mk_vorbis params =
               { f with Encoder.Vorbis.mode = Encoder.Vorbis.VBR q }
           | ("channels",{ term = Int i }) ->
               { f with Encoder.Vorbis.channels = i }
+          | ("bytes_per_page",{ term = Int i }) ->
+              { f with Encoder.Vorbis.fill = Some i }
           | ("",{ term = Var s }) when String.lowercase s = "mono" ->
               { f with Encoder.Vorbis.channels = 1 }
           | ("",{ term = Var s }) when String.lowercase s = "stereo" ->
@@ -479,6 +684,7 @@ let mk_theora params =
     {
       Encoder.Theora.
        bitrate_control    = Encoder.Theora.Quality 40 ;
+       fill               = None ;
        width              = Frame.video_width ;
        height             = Frame.video_height ;
        picture_width      = Frame.video_width ;
@@ -511,14 +717,14 @@ let mk_theora params =
           | ("width",({ term = Int i } as t)) ->
               (* According to the doc: must be a multiple of 16, and less than 1048576. *)
               if i mod 16 <> 0 || i >= 1048576 then
-                raise (Error (t,"invalid frame width value")) ;
+                raise (Error (t,"invalid frame width value (should be a multiple of 16)")) ;
               { f with Encoder.Theora.
                     width = Lazy.lazy_from_val i;
                     picture_width = Lazy.lazy_from_val i }
           | ("height",({ term = Int i } as t)) ->
               (* According to the doc: must be a multiple of 16, and less than 1048576. *)
               if i mod 16 <> 0 || i >= 1048576 then
-                raise (Error (t,"invalid frame height value")) ;
+                raise (Error (t,"invalid frame height value (should be a multiple of 16)")) ;
               { f with Encoder.Theora.
                     height = Lazy.lazy_from_val i;
                     picture_height = Lazy.lazy_from_val i }
@@ -572,6 +778,8 @@ let mk_theora params =
               { f with Encoder.Theora.buffer_delay = Some i }
           | ("speed",{ term = Int i }) ->
               { f with Encoder.Theora.speed = Some i }
+          | ("bytes_per_page",{ term = Int i }) ->
+              { f with Encoder.Theora.fill = Some i }
           | (_,t) -> raise (generic_error t))
       defaults params
   in
@@ -582,6 +790,7 @@ let mk_dirac params =
     {
       Encoder.Dirac.
        quality            = 35. ;
+       fill               = None ;
        width              = Frame.video_width ;
        height             = Frame.video_height ;
        aspect_numerator   = 1 ;
@@ -604,6 +813,8 @@ let mk_dirac params =
               { f with Encoder.Dirac.aspect_numerator = i }
           | ("aspect_denominator",{ term = Int i }) ->
               { f with Encoder.Dirac.aspect_denominator = i }
+          | ("bytes_per_page",{ term = Int i }) ->
+              { f with Encoder.Dirac.fill = Some i }
           | (_,t) -> raise (generic_error t))
       defaults params
   in
@@ -613,11 +824,14 @@ let mk_speex params =
   let defaults =
     { Encoder.Speex.
         stereo = false ;
+        fill = None ;
         samplerate = 44100 ;
         bitrate_control = Encoder.Speex.Quality 7;
         mode = Encoder.Speex.Narrowband ;
         frames_per_packet = 1 ;
-        complexity = None
+        complexity = None ;
+        dtx = false ;
+        vad = false
     }
   in
   let speex =
@@ -661,6 +875,12 @@ let mk_speex params =
               if i < 1 || i > 10 then
                 raise (Error (t,"Speex complexity should be in 1..10"));
               { f with Encoder.Speex.complexity = Some i }
+          | ("bytes_per_page",{ term = Int i }) ->
+              { f with Encoder.Speex.fill = Some i }
+          | ("dtx", { term = Bool b }) ->
+              { f with Encoder.Speex.dtx = b }
+          | ("vad", { term = Bool b }) ->
+              { f with Encoder.Speex.vad = b }
           | ("",{ term = Var s }) when String.lowercase s = "mono" ->
               { f with Encoder.Speex.stereo = false }
           | ("",{ term = Var s }) when String.lowercase s = "stereo" ->
@@ -669,3 +889,61 @@ let mk_speex params =
       defaults params
   in
     Encoder.Ogg.Speex speex
+
+let mk_gstreamer params =
+  let defaults =
+    { Encoder.GStreamer.
+       channels  = 2;
+       audio     = Some "lamemp3enc";
+       has_video = true;
+       video     = Some "x264enc";
+       muxer     = Some "mpegtsmux";
+       metadata  = "metadata";
+       log        = 5;
+       pipeline  = None
+    }
+  in
+  let gstreamer =
+    let perhaps = function
+      | "" -> None
+      | s  -> Some s
+    in
+    List.fold_left
+      (fun f ->
+        function
+          | ("channels",{ term = Int i }) ->
+              { f with Encoder.GStreamer.channels = i }
+          | ("audio",{ term = String s }) ->
+              { f with Encoder.GStreamer.audio = perhaps s }
+          | ("has_video",{ term = Bool b }) ->
+              { f with Encoder.GStreamer.has_video = b }
+          | ("video",{ term = String s }) ->
+              { f with Encoder.GStreamer.video = perhaps s }
+          | ("muxer",{ term = String s }) ->
+              { f with Encoder.GStreamer.muxer = perhaps s }
+          | ("metadata",{ term = String s }) ->
+              { f with Encoder.GStreamer.metadata = s }
+          | ("log",{ term = Int i }) ->
+              { f with Encoder.GStreamer.log = i }
+          | ("pipeline",{ term = String s }) ->
+              { f with Encoder.GStreamer.pipeline = perhaps s }
+          | (_,t) -> raise (generic_error t))
+      defaults params
+  in
+  let ret = mk (Encoder (Encoder.GStreamer gstreamer)) in
+    if gstreamer.Encoder.GStreamer.pipeline = None &&
+       gstreamer.Encoder.GStreamer.audio <> None && 
+       gstreamer.Encoder.GStreamer.channels = 0
+    then
+      raise
+        (Error (ret, "Must have at least one audio channel when \
+                         passing an audio pipeline!"));
+    if gstreamer.Encoder.GStreamer.pipeline = None &&
+       gstreamer.Encoder.GStreamer.video <> None && 
+       gstreamer.Encoder.GStreamer.audio <> None && 
+       gstreamer.Encoder.GStreamer.muxer = None
+    then
+      raise
+        (Error (ret, "Must have a muxer when passing an audio and \
+                         a video pipeline!"));
+    ret
