@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,6 +23,12 @@
 open Dtools
 open Printf
 
+(** Runner module signature. *)
+module type Runner_t =
+sig
+  val options : (string list * Arg.spec * string) list
+end
+
 let usage =
   "Usage : liquidsoap [OPTION, SCRIPT or EXPR]...\n\
   \ - SCRIPT for evaluating a liquidsoap script file;\n\
@@ -114,29 +120,13 @@ let load_libs () =
   do_eval ~lib:true ;
   load_libs ()
 
-let plugin_doc name =
+let lang_doc name =
   secondary_task := true ;
   load_libs () ;
-  let found = ref false in
-    List.iter
-      (fun (lbl, i) ->
-         match
-           try Some (i#get_subsection name) with Not_found -> None
-         with
-           | None -> ()
-           | Some s ->
-               found := true ;
-               Printf.printf "*** One entry in %s:\n" lbl ;
-               let print =
-                 if lbl="scripting values" then
-                   Doc.print_lang
-                 else
-                   Doc.print
-               in
-                 print s)
-      Plug.plugs#get_subsections ;
-    if not !found then
-      Printf.printf "Plugin not found!\n%!"
+  try
+    Doc.print_lang (Lang_values.builtins#get_subsection name)
+  with
+    | Not_found -> Printf.printf "Plugin not found!\n%!"
 
 let process_request s =
   load_libs () ;
@@ -235,7 +225,7 @@ struct
       let subs =
         List.map (function s -> aux (level+1) (p s) (t#path [s])) t#subs
       in
-      let title,default,set,comment = 
+      let title,default,set,comment =
         if liqi then
           Printf.sprintf "h%d. %s\n",
           Printf.sprintf "Default: @%s@\n",
@@ -243,7 +233,7 @@ struct
                           set(%S,%s)\n\
                           %%%%\n",
           (fun l -> String.concat ""
-                     (List.map (fun s -> Printf.sprintf "%s\n" s) l)) 
+                     (List.map (fun s -> Printf.sprintf "%s\n" s) l))
         else
           (fun _ ->
              if t#kind = None then
@@ -333,7 +323,149 @@ let format_doc s =
 
 let log = Log.make ["main"]
 
-let options =
+let options = [
+    ["-"],
+    Arg.Unit (fun () -> eval `StdIn),
+    "Read script from standard input." ;
+
+    ["-r"],
+    Arg.String process_request,
+    "Process a request." ;
+
+    ["-h"],
+    Arg.String lang_doc,
+    "Get help about a scripting value: \
+     source, operator, builtin or library function, etc.";
+
+    ["-c";"--check"],
+    Arg.Unit (fun () ->
+                secondary_task := true ;
+                dont_run := true),
+    "Check and evaluate scripts but do not perform any streaming." ;
+
+    ["-cl";"--check-lib"],
+    Arg.Unit (fun () ->
+                last_item_lib := true ;
+                secondary_task := true ;
+                dont_run := true),
+    "Like --check but treats all scripts and expressions as libraries, \
+     so that unused toplevel variables are not reported." ;
+
+    ["-p";"--parse-only"],
+    Arg.Unit (fun () ->
+                secondary_task := true ;
+                parse_only := true),
+    "Parse scripts but do not type-check and run them." ;
+
+    ["-q";"--quiet"],
+    Arg.Unit (fun () -> Log.conf_stdout#set false),
+    "Do not print log messages on standard output." ;
+
+    ["-v";"--verbose"],
+    Arg.Unit (fun () -> Log.conf_stdout#set true),
+    "Print log messages on standard output." ;
+
+    ["-f";"--force-start"],
+    Arg.Unit (fun () -> force_start#set true),
+    "For advanced dynamic uses: force liquidsoap to start \
+     even when no active source is initially defined." ;
+
+    ["--debug"],
+    Arg.Unit (fun () -> Log.conf_level#set (max 4 Log.conf_level#get)),
+    "Print debugging log messages." ]
+    @
+    (if Configure.dynlink then
+        [["--dynamic-plugins-dir"],
+            Arg.String (fun d ->
+            Dyntools.load_plugins_dir d),
+         "Directory where to look for plugins."]
+      else
+        [])
+    @
+    [["--errors-as-warnings"],
+    Arg.Set Lang_values.errors_as_warnings,
+    "Issue warnings instead of fatal errors for unused variables \
+     and ignored expressions. If you are not sure about it, it is better \
+     to not use it." ]
+    @
+    (* Unix.fork is not implemented in Win32. *)
+    (if Sys.os_type <> "Win32" then
+      [["-d";"--daemon"],
+       Arg.Unit (fun f -> Init.conf_daemon#set true),
+       "Run in daemon mode."]
+     else [])
+    @
+    [["-t";"--enable-telnet"],
+    Arg.Unit (fun _ -> Server.conf_telnet#set true),
+    "Enable the telnet server." ;
+
+    ["-T";"--disable-telnet"],
+    Arg.Unit (fun _ -> Server.conf_telnet#set false),
+    "Disable the telnet server." ;
+
+    ["-u";"--enable-unix-socket"],
+    Arg.Unit (fun _ -> Server.conf_socket#set true),
+    "Enable the unix socket." ;
+
+    ["-U";"--disable-unix-socket"],
+    Arg.Unit (fun _ -> Server.conf_socket#set false),
+    "Disable the unix socket." ;
+
+    ["--list-plugins-xml"],
+    Arg.Unit (fun () ->
+                secondary_task := true ;
+                load_libs () ;
+                Doc.print_xml (Plug.plugs:Doc.item)),
+    Printf.sprintf
+      "List all plugins (builtin scripting values, \
+       supported formats and protocols), \
+       output as XML." ;
+
+    ["--list-plugins"],
+    Arg.Unit (fun () ->
+                secondary_task := true ;
+                load_libs () ;
+                Doc.print (Plug.plugs:Doc.item)),
+    Printf.sprintf
+      "List all plugins (builtin scripting values, \
+       supported formats and protocols)." ;
+
+    ["--no-pervasives"],
+    Arg.Clear pervasives,
+    Printf.sprintf
+      "Do not load pervasives script libraries (i.e., %s/*.liq)."
+      Configure.libs_dir ;
+
+    ["-i"],
+    Arg.Set Configure.display_types,
+    "Display infered types." ;
+
+    ["--version"],
+    Arg.Unit (fun () ->
+                Printf.printf
+                  "Liquidsoap %s%s\n\
+                   Copyright (c) 2003-2013 Savonet team\n\
+                   Liquidsoap is open-source software, \
+                   released under GNU General Public License.\n\
+                   See <http://liquidsoap.fm> for more information.\n"
+                   Configure.version SVN.rev ;
+                exit 0),
+    "Display liquidsoap's version." ;
+
+    ["--interactive"],
+    Arg.Set interactive,
+    "Start an interactive interpreter." ;
+
+    ["--"],
+    Arg.Unit (fun () -> Arg.current := Array.length Shebang.argv - 1),
+    "Stop parsing the command-line and pass subsequent items to the script."
+
+    ] @ (LiqConf.args Configure.conf)
+
+let expand_options options =
+  let options =
+    List.sort (fun (x,_,_) (y,_,_) -> compare x y)  options
+  in
   List.fold_left
     (fun l (la,b,c) ->
         let ta = List.hd (List.rev la) in
@@ -343,293 +475,156 @@ let options =
                        if a = ta then "\n" ^ format_doc c else ""))
             la
         in
-          l@expand) []
-    (let opts = [
-      ["-"],
-      Arg.Unit (fun () -> eval `StdIn),
-      "Read script from standard input." ;
+        l@expand) [] options
 
-      ["-r"],
-      Arg.String process_request,
-      "Process a request." ;
+module Make(Runner : Runner_t) =
+struct
+  let () =
+    log#f 3 "Liquidsoap %s%s" Configure.version SVN.rev ;
+    log#f 3 "Using:%s" Configure.libs_versions ;
+    if Configure.scm_snapshot then
+      List.iter (log#f 2 "%s")
+        ["";
+         "DISCLAIMER: This version of Liquidsoap has been";
+         "compiled from a snapshot of the development code.";
+         "As such, it should not be used in production";
+         "unless you know what you are doing!";
+         "";
+         "We are, however, very interested in any feedback";
+         "about our development code and committed to fix";
+         "issues as soon as possible.";
+         "";
+         "If you are interested in collaborating to";
+         "the development of Liquidsoap, feel free to";
+         "drop us a mail at <savonet-devl@lists.sf.net>";
+         "or to join the #savonet IRC channel on Freenode.";
+         "";
+         "Please send any bug report or feature request";
+         "at <https://github.com/savonet/liquidsoap/issues>.";
+         "";
+         "We hope you enjoy this snapshot build of Liquidsoap!";
+         ""]
 
-      ["-h"],
-      Arg.String plugin_doc,
-      "Print the description of a plugin.";
+  (** Just like Arg.parse_argv but with Arg.parse's behavior on errors.. *)
+  let parse argv l f msg =
+    try
+      Arg.parse_argv argv l f msg ;
+    with
+      | Arg.Bad msg -> Printf.eprintf "%s" msg ; exit 2
+      | Arg.Help msg -> Printf.printf "%s" msg ; exit 0
 
-      ["-c";"--check"],
-      Arg.Unit (fun () ->
-                  secondary_task := true ;
-                  dont_run := true),
-      "Check and evaluate scripts but do not perform any streaming." ;
-
-      ["-cl";"--check-lib"],
-      Arg.Unit (fun () ->
-                  last_item_lib := true ;
-                  secondary_task := true ;
-                  dont_run := true),
-      "Like --check but treats all scripts and expressions as libraries, \
-       so that unused toplevel variables are not reported." ;
-
-      ["-p";"--parse-only"],
-      Arg.Unit (fun () ->
-                  secondary_task := true ;
-                  parse_only := true),
-      "Parse scripts but do not type-check and run them." ;
-
-      ["-q";"--quiet"],
-      Arg.Unit (fun () -> Log.conf_stdout#set false),
-      "Do not print log messages on standard output." ;
-
-      ["-v";"--verbose"],
-      Arg.Unit (fun () -> Log.conf_stdout#set true),
-      "Print log messages on standard output." ;
-
-      ["-f";"--force-start"],
-      Arg.Unit (fun () -> force_start#set true),
-      "For advanced dynamic uses: force liquidsoap to start \
-       even when no active source is initially defined." ;
-
-      ["--debug"],
-      Arg.Unit (fun () -> Log.conf_level#set (max 4 Log.conf_level#get)),
-      "Print debugging log messages." ]
-      @
-      (if Configure.dynlink then
-          [["--dynamic-plugins-dir"],
-              Arg.String (fun d ->
-              Dyntools.load_plugins_dir d),
-           "Directory where to look for plugins."]
-        else
-          [])
-      @ 
-      [["--errors-as-warnings"],
-      Arg.Set Lang_values.errors_as_warnings,
-      "Issue warnings instead of fatal errors for unused variables \
-       and ignored expressions. If you are not sure about it, it is better \
-       to not use it." ]
-      @
-      (* Unix.fork is not implemented in Win32. *) 
-      (if Sys.os_type <> "Win32" then
-        [["-d";"--daemon"],
-         Arg.Unit (fun f -> Init.conf_daemon#set true),
-         "Run in daemon mode."]
-       else [])
-      @
-      [["-t";"--enable-telnet"],
-      Arg.Unit (fun _ -> Server.conf_telnet#set true),
-      "Enable the telnet server." ;
-
-      ["-T";"--disable-telnet"],
-      Arg.Unit (fun _ -> Server.conf_telnet#set false),
-      "Disable the telnet server." ;
-
-      ["-u";"--enable-unix-socket"],
-      Arg.Unit (fun _ -> Server.conf_socket#set true),
-      "Enable the unix socket." ;
-
-      ["-U";"--disable-unix-socket"],
-      Arg.Unit (fun _ -> Server.conf_socket#set false),
-      "Disable the unix socket." ;
-
-      ["--list-plugins-xml"],
-      Arg.Unit (fun () ->
-                  secondary_task := true ;
-                  load_libs () ;
-                  Doc.print_xml (Plug.plugs:Doc.item)),
-      Printf.sprintf
-        "List all plugins (builtin scripting values, \
-         supported formats and protocols), \
-         output as XML." ;
-
-      ["--list-plugins"],
-      Arg.Unit (fun () ->
-                  secondary_task := true ;
-                  load_libs () ;
-                  Doc.print (Plug.plugs:Doc.item)),
-      Printf.sprintf
-        "List all plugins (builtin scripting values, \
-         supported formats and protocols)." ;
-
-      ["--no-pervasives"],
-      Arg.Clear pervasives,
-      Printf.sprintf
-        "Do not load pervasives script libraries (i.e., %s/*.liq)."
-        Configure.libs_dir ;
-
-      ["-i"],
-      Arg.Set Configure.display_types,
-      "Display infered types." ;
-
-      ["--version"],
-      Arg.Unit (fun () ->
-                  Printf.printf
-                    "Liquidsoap %s%s\n\
-                     Copyright (c) 2003-2011 Savonet team\n\
-                     Liquidsoap is open-source software, \
-                     released under GNU General Public License.\n\
-                     See <http://savonet.sf.net> for more information.\n"
-                    Configure.version SVN.rev ;
-                  exit 0),
-      "Display liquidsoap's version." ;
-
-      ["--interactive"],
-      Arg.Set interactive,
-      "Start an interactive interpreter." ;
-
-      ["--"],
-      Arg.Unit (fun () -> Arg.current := Array.length Shebang.argv - 1),
-      "Stop parsing the command-line and pass subsequent items to the script."
-
-      ]
-
-     in 
-     let opts = 
-       List.sort (fun (x,_,_) (y,_,_) -> compare x y) opts
-     in opts@LiqConf.args Configure.conf)
-
-let () = 
-  log#f 3 "Liquidsoap %s%s" Configure.version SVN.rev ;
-  log#f 3 "Using:%s" Configure.libs_versions ;
-  if Configure.svn_snapshot then
-    List.iter (log#f 2 "%s")
-      ["";
-       "DISCLAIMER: This version of Liquidsoap has been";
-       "compiled from a snapshot of the development code.";
-       "As such, it should not be used in production";
-       "unless you know what you are doing!";
-       "";
-       "We are, however, very interested in any feedback";
-       "about our development code and committed to fix";
-       "issues as soon as possible.";
-       "";
-       "If you are interested in collaborating to";
-       "the development of Liquidsoap, feel free to";
-       "drop us a mail at <savonet-devl@lists.sf.net>";
-       "or to join the #savonet IRC channel on Freenode.";
-       "";
-       "Please send any bug report or feature request";
-       "at <http://dev.sourcefabric.org/browse/LS>.";
-       "";
-       "We hope you enjoy this snapshot build of Liquidsoap!";
-       ""]
-
-(** Just like Arg.parse_argv but with Arg.parse's behavior on errors.. *)
-let parse argv l f msg =
-  try
-    Arg.parse_argv argv l f msg ;
-  with
-    | Arg.Bad msg -> Printf.eprintf "%s" msg ; exit 2
-    | Arg.Help msg -> Printf.printf "%s" msg ; exit 0
-
-let absolute s =
-  if String.length s > 0 && s.[0] <> '/' then
-    (Unix.getcwd ())^"/"^s
-  else
-    s
-
-(* Load plugins and dynamic loaded libraries: 
- * these should be loaded as early as possible since we want to be
- * able to use them in scripts... *)
-let () =
-  Configure.load_dynlinks ();
-  Configure.load_plugins_dir Configure.plugins_dir
-
-(* Startup *)
-let () =
-
-  Random.self_init () ;
-
-  (* Set the default values. *)
-  Log.conf_file_path#set_d (Some "<syslogdir>/<script>.log") ;
-  Init.conf_daemon_pidfile#set_d (Some true) ;
-  Init.conf_daemon_pidfile_path#set_d (Some "<sysrundir>/<script>.pid") ;
-
-  (* We only allow evaluation of
-   * lazy configuration keys now. *)
-  Frame.allow_lazy_config_eval ();
-
-  (* Parse command-line, and notably load scripts. *)
-  parse Shebang.argv options (fun s -> eval (`Expr_or_File s)) usage ;
-  do_eval ~lib:!last_item_lib
-
-(* When the log/pid paths have their definitive values,
- * expand substitutions and check directories.
- * This should be ran just before Dtools init. *)
-let check_directories () =
-
-  (* Now that the paths have their definitive value, expand <shortcuts>. *)
-  let subst conf = conf#set (Configure.subst_vars conf#get) in
-  subst Log.conf_file_path ;
-  subst Init.conf_daemon_pidfile_path ;
-
-  let check_dir conf_path kind =
-    let path = conf_path#get in
-    let dir = Filename.dirname path in
-      if not (Sys.file_exists dir) then
-        let routes = Configure.conf#routes conf_path#ut in
-          Printf.printf
-            "FATAL ERROR: %s directory %S does not exist.\n\
-             To change it, add the following to your script:\n\
-            \  set(%S, \"<path>\")\n"
-            kind dir (Conf.string_of_path (List.hd routes)) ;
-          exit 1
-  in
-    if Log.conf_file#get then
-      check_dir Log.conf_file_path "Log" ;
-    if Init.conf_daemon#get && Init.conf_daemon_pidfile#get then
-      check_dir Init.conf_daemon_pidfile_path "PID"
-
-(* Now that outputs have been defined, we can start the main loop. *)
-let () =
-  let cleanup () =
-    log#f 3 "Shutdown started!" ;
-    Clock.stop () ;
-    log#f 3 "Waiting for threads to terminate..." ;
-    Tutils.join_all () ;
-    log#f 3 "Cleaning downloaded files..." ;
-    Request.clean ()
-  in
-  let main () =
-    (* See http://caml.inria.fr/mantis/print_bug_page.php?bug_id=4640
-     * for this: we want Unix EPIPE error and not SIGPIPE, which
-     * crashes the program.. *)
-    if Sys.os_type <> "Win32" then
-     begin
-      Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
-      ignore (Unix.sigprocmask Unix.SIG_BLOCK [Sys.sigpipe])
-     end;
-    (* On Windows we need to initiate shutdown ourselves by catching INT
-     * since dtools doesn't do it. *)
-    if Sys.os_type = "Win32" then
-      Sys.set_signal Sys.sigint
-        (Sys.Signal_handle (fun _ -> Tutils.shutdown ())) ;
-    (* TODO if start fails (e.g. invalid password or mountpoint) it
-     *   raises an exception and dtools catches it so we don't get
-     *   a backtrace (by default at least). *)
-    Clock.start () ;
-    Tutils.main ()
-  in
-    ignore (Init.at_stop cleanup) ;
-    if !interactive then begin
-      Log.conf_stdout#set_d (Some false) ;
-      Log.conf_file#set_d (Some true) ;
-      let default_log =
-        Filename.temp_file
-          (Printf.sprintf "liquidsoap-%d-" (Unix.getpid ())) ".log"
-      in
-      Log.conf_file_path#set_d (Some default_log) ;
-      ignore (Init.at_stop (fun _ -> Sys.remove default_log)) ;
-      check_directories () ;
-      ignore (Thread.create Lang.interactive ()) ;
-      Init.init main
-    end else if Source.has_outputs () || force_start#get then
-      if not !dont_run then begin
-        check_directories () ;
-        Init.init ~prohibit_root:true main
-      end else
-        cleanup ()
+  let absolute s =
+    if String.length s > 0 && s.[0] <> '/' then
+      (Unix.getcwd ())^"/"^s
     else
-      (* If there's no output and no secondary task has been performed,
-       * warn the user that his scripts didn't define any output. *)
-      if not !secondary_task then
-        Printf.printf "No output defined, nothing to do.\n"
+      s
+
+  (* Load plugins and dynamic loaded libraries:
+   * these should be loaded as early as possible since we want to be
+   * able to use them in scripts... *)
+  let () =
+    Configure.load_dynlinks ();
+    Configure.load_plugins_dir Configure.plugins_dir
+
+  (* Startup *)
+  let () =
+    Random.self_init () ;
+
+    (* Set the default values. *)
+    Log.conf_file_path#set_d (Some "<syslogdir>/<script>.log") ;
+    Init.conf_daemon_pidfile#set_d (Some true) ;
+    Init.conf_daemon_pidfile_path#set_d (Some "<sysrundir>/<script>.pid") ;
+
+    (* We only allow evaluation of
+     * lazy configuration keys now. *)
+    Frame.allow_lazy_config_eval ();
+
+    (* Parse command-line, and notably load scripts. *)
+    parse Shebang.argv (expand_options Runner.options) (fun s -> eval (`Expr_or_File s)) usage ;
+    do_eval ~lib:!last_item_lib
+
+  (* When the log/pid paths have their definitive values,
+   * expand substitutions and check directories.
+   * This should be ran just before Dtools init. *)
+  let check_directories () =
+
+    (* Now that the paths have their definitive value, expand <shortcuts>. *)
+    let subst conf = conf#set (Configure.subst_vars conf#get) in
+    subst Log.conf_file_path ;
+    subst Init.conf_daemon_pidfile_path ;
+
+    let check_dir conf_path kind =
+      let path = conf_path#get in
+      let dir = Filename.dirname path in
+        if not (Sys.file_exists dir) then
+          let routes = Configure.conf#routes conf_path#ut in
+            Printf.printf
+              "FATAL ERROR: %s directory %S does not exist.\n\
+               To change it, add the following to your script:\n\
+              \  set(%S, \"<path>\")\n"
+              kind dir (Conf.string_of_path (List.hd routes)) ;
+            exit 1
+    in
+      if Log.conf_file#get then
+        check_dir Log.conf_file_path "Log" ;
+      if Init.conf_daemon#get && Init.conf_daemon_pidfile#get then
+        check_dir Init.conf_daemon_pidfile_path "PID"
+
+  (* Now that outputs have been defined, we can start the main loop. *)
+  let () =
+    let cleanup () =
+      log#f 3 "Shutdown started!" ;
+      Clock.stop () ;
+      log#f 3 "Waiting for threads to terminate..." ;
+      Tutils.join_all () ;
+      log#f 3 "Cleaning downloaded files..." ;
+      Request.clean () ;
+      log#f 3 "Freeing memory..." ;
+      Gc.full_major ()
+    in
+    let main () =
+      (* See http://caml.inria.fr/mantis/print_bug_page.php?bug_id=4640
+       * for this: we want Unix EPIPE error and not SIGPIPE, which
+       * crashes the program.. *)
+      if Sys.os_type <> "Win32" then
+       begin
+        Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+        ignore (Unix.sigprocmask Unix.SIG_BLOCK [Sys.sigpipe])
+       end;
+      (* On Windows we need to initiate shutdown ourselves by catching INT
+       * since dtools doesn't do it. *)
+      if Sys.os_type = "Win32" then
+        Sys.set_signal Sys.sigint
+          (Sys.Signal_handle (fun _ -> Tutils.shutdown ())) ;
+      (* TODO if start fails (e.g. invalid password or mountpoint) it
+       *   raises an exception and dtools catches it so we don't get
+       *   a backtrace (by default at least). *)
+      Clock.start () ;
+      Tutils.main ()
+    in
+      ignore (Init.at_stop cleanup) ;
+      if !interactive then begin
+        load_libs () ;
+        Log.conf_stdout#set_d (Some false) ;
+        Log.conf_file#set_d (Some true) ;
+        let default_log =
+          Filename.temp_file
+            (Printf.sprintf "liquidsoap-%d-" (Unix.getpid ())) ".log"
+        in
+        Log.conf_file_path#set_d (Some default_log) ;
+        ignore (Init.at_stop (fun _ -> Sys.remove default_log)) ;
+        check_directories () ;
+        ignore (Thread.create Lang.interactive ()) ;
+        Init.init main
+      end else if Source.has_outputs () || force_start#get then
+        if not !dont_run then begin
+          check_directories () ;
+          Init.init ~prohibit_root:true main
+        end else
+          cleanup ()
+      else
+        (* If there's no output and no secondary task has been performed,
+         * warn the user that his scripts didn't define any output. *)
+        if not !secondary_task then
+          Printf.printf "No output defined, nothing to do.\n"
+end
