@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,22 +28,20 @@ let pi = 3.14159265358979323846
 type filter_type = Band_stop | Band_pass | High_pass | Low_pass | All_pass
 type filter_family = Butterworth | Resonator
 
-class iir (source:source) filter_family filter_type order
-  freq1 freq2 qfactor debug =
+class iir ~kind (source:source)
+          filter_family filter_type order freq1 freq2 qfactor =
+  let channels = (Frame.type_of_kind kind).Frame.audio in
+  let rate = float (Frame.audio_of_seconds 1.) in
 object (self)
-  inherit operator [source] as super
-  
+  inherit operator ~name:"iir_filter" kind [source] as super
+
   (* Params *)
-  val raw_alpha1 = freq1 /. float_of_int (Fmt.samples_per_second ())
-  val raw_alpha2 = freq2 /. float_of_int (Fmt.samples_per_second ())
-  val warped_alpha1 = tan(pi *. freq1
-                          /. float_of_int (Fmt.samples_per_second ()))
-                        /. pi
-  val warped_alpha2 = tan(pi *. freq2
-                          /. float_of_int (Fmt.samples_per_second ()))
-                        /. pi
+  val raw_alpha1 = freq1 /. rate
+  val raw_alpha2 = freq2 /. rate
+  val warped_alpha1 = tan (pi *. freq1 /. rate) /. pi
+  val warped_alpha2 = tan (pi *. freq2 /. rate) /. pi
   val mutable gain = 0.
-  
+
   (* Used for computation *)
   val mutable polemask = 0
   val mutable splane_poles = [||]
@@ -59,26 +57,23 @@ object (self)
   val mutable dc_gain = {re = 0. ; im = 0.}
   val mutable fc_gain = {re = 0. ; im = 0.}
   val mutable hf_gain = {re = 0. ; im = 0.}
-  
+
   (* Coefficients *)
   val mutable xcoeffs = [||]
   val mutable ycoeffs = [||]
-  
+
   (* I/O shift registries *)
   val mutable xv = [||]
   val mutable yv = [||]
-  
+
   initializer
-    if debug then
-      (
-        Printf.printf "================== BEG INIT FILTER %s ==================\n" self#id ;
-        Printf.printf "Alpha 1: %.013f (warped: %.013f)\n" raw_alpha1 warped_alpha1 ;
-        Printf.printf "Alpha 2: %.013f (warped: %.013f)\n" raw_alpha2 warped_alpha2 ;
-        Printf.printf "Q: %.013f\n" qfactor
-      ) ;
+    self#log#f 4 "Initializing..." ;
+    self#log#f 4 "Alpha 1: %.013f (warped: %.013f)" raw_alpha1 warped_alpha1 ;
+    self#log#f 4 "Alpha 2: %.013f (warped: %.013f)" raw_alpha2 warped_alpha2 ;
+    self#log#f 4 "Q: %.013f" qfactor ;
     let cor a = {re = a ; im = 0.}
-      and ( +~ ), ( -~ ), ( *~ ), ( /~ ) =
-        Complex.add, Complex.sub, Complex.mul, Complex.div
+    and ( +~ ), ( -~ ), ( *~ ), ( /~ ) =
+      Complex.add, Complex.sub, Complex.mul, Complex.div
     in
     let multin w s n c =
       (* multiply factor (z-w) into coeffs *)
@@ -113,26 +108,21 @@ object (self)
       (* evaluate response, substituting for z *)
       Complex.div (eval t nz z) (eval b np z)
     in
-      (match filter_family with
+      begin match filter_family with
         | Butterworth ->
             (* Compute S-plane poles (Butterworth) *)
-            if debug then
-              Printf.printf "This is a Butterworth filter.\n" ;
+            self#log#f 4 "This is a Butterworth filter." ;
             let choosepole z =
-              if (z.re < 0.) then
-                (
-                  if debug then
-                    Printf.printf "z = %+.013f %+.013f i.\n" z.re z.im ;
-                  if (polemask mod 2 == 0) then
-                    (
-                      splane_poles <- Array.append splane_poles [|z|] ;
-                      splane_numpoles <- splane_numpoles + 1
-                    ) ;
-                  polemask <- polemask lsl 1
-                ) ;
+              if (z.re < 0.) then begin
+                self#log#f 4 "z = %+.013f %+.013f i." z.re z.im ;
+                if polemask mod 2 == 0 then begin
+                  splane_poles <- Array.append splane_poles [|z|] ;
+                  splane_numpoles <- splane_numpoles + 1
+                end ;
+                polemask <- polemask lsl 1
+              end
             in
-              if debug then
-                Printf.printf "Theta:\n" ;
+              self#log#f 4 "Theta:" ;
               for i = 0 to (2 * order) - 1 do
                 let theta =
                   match (order mod 2) with
@@ -141,19 +131,18 @@ object (self)
                         ((float_of_int i +. 0.5) *. pi) /. (float_of_int order)
                     | _ -> assert false
                 in
-                  if debug then
-                    Printf.printf "%d: %+.013f --> %+.013f %+.013f i.\n" i theta (cos theta) (sin theta) ;
+                  self#log#f 4
+                    "%d: %+.013f --> %+.013f %+.013f i."
+                    i theta (cos theta) (sin theta) ;
                   choosepole ({re = cos(theta) ; im = sin(theta)})
               done;
             (* Normalize *)
-            let w1 = cor (2. *. pi *. warped_alpha1)
-              and w2 = cor (2. *. pi *. warped_alpha2)
-            in
-            ( match filter_type with
+            let w1 = cor (2. *. pi *. warped_alpha1) in
+            let w2 = cor (2. *. pi *. warped_alpha2) in
+              begin match filter_type with
                 | Band_stop ->
                     (* Band-stop filter *)
-                    if debug then
-                      Printf.printf "This is a band-stop filter.\n" ;
+                    self#log#f 4 "This is a band-stop filter." ;
                     let w0 = sqrt (w1 *~ w2)
                       and bw = w2 -~ w1
                     in
@@ -174,14 +163,11 @@ object (self)
                       done;
                       splane_numpoles <- splane_numpoles * 2 ;
                       splane_numzeros <- splane_numpoles ;
-                      xv <- Array.make_matrix (Fmt.channels ())
-                        (order * 2 + 1) 0. ;
-                      yv <- Array.make_matrix (Fmt.channels ())
-                        (order * 2 + 1) 0.
+                      xv <- Array.make_matrix channels (order * 2 + 1) 0. ;
+                      yv <- Array.make_matrix channels (order * 2 + 1) 0.
                 | Band_pass ->
                     (* Band-pass filter *)
-                    if debug then
-                      Printf.printf "This is a band-pass filter.\n" ;
+                    self#log#f 4 "This is a band-pass filter." ;
                     let w0 = sqrt (w1 *~ w2)
                       and bw = w2 -~ w1
                     in
@@ -200,14 +186,11 @@ object (self)
                       done;
                       splane_numzeros <- splane_numpoles ;
                       splane_numpoles <- splane_numpoles * 2 ;
-                      xv <- Array.make_matrix (Fmt.channels ())
-                        (order * 2 + 1) 0. ;
-                      yv <- Array.make_matrix (Fmt.channels ())
-                        (order * 2 + 1) 0.
+                      xv <- Array.make_matrix channels (order * 2 + 1) 0. ;
+                      yv <- Array.make_matrix channels (order * 2 + 1) 0.
                 | High_pass ->
                     (* Hi-pass filter *)
-                    if debug then
-                      Printf.printf "This is a hi-pass filter.\n" ;
+                    self#log#f 4 "This is a hi-pass filter." ;
                     for i = 0 to splane_numpoles - 1 do
                       splane_poles.(i) <- w1 /~ splane_poles.(i) ;
                     done;
@@ -216,104 +199,124 @@ object (self)
                       splane_zeros <- Array.append splane_zeros [|cor 0.|] ;
                     done;
                     splane_numzeros <- splane_numpoles ;
-                    xv <- Array.make_matrix (Fmt.channels ()) (order + 1) 0. ;
-                    yv <- Array.make_matrix (Fmt.channels ()) (order + 1) 0.
+                    xv <- Array.make_matrix channels (order + 1) 0. ;
+                    yv <- Array.make_matrix channels (order + 1) 0.
                 | Low_pass ->
                     (* Lo-pass filter *)
-                    if debug then
-                      Printf.printf "This is a lo-pass filter.\n" ;
+                    self#log#f 4 "This is a lo-pass filter." ;
                     for i = 0 to splane_numpoles - 1 do
                       splane_poles.(i) <- splane_poles.(i) *~ w1 ;
                     done;
                     splane_numzeros <- 0 ;
-                    xv <- Array.make_matrix (Fmt.channels ()) (order + 1) 0. ;
-                    yv <- Array.make_matrix (Fmt.channels ()) (order + 1) 0.
-                | _ -> assert false ) ;
-            (* Compute Z-plane zeros & poles using bilinear transform *)
-            if debug then
-              (
-                Printf.printf "S-Plane zeros:\n%s\n" (String.concat "\n" (Array.to_list ((Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im) splane_zeros)))) ;
-                Printf.printf "S-Plane poles:\n%s\n" (String.concat "\n" (Array.to_list ((Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im) splane_poles))))
-              ) ;
-            zplane_numpoles <- splane_numpoles ;
-            zplane_numzeros <- splane_numzeros ;
-            let blt a = Complex.div (Complex.add {re = 2. ; im = 0.} a)
-                                    (Complex.sub {re = 2. ; im = 0.} a)
-            in
-              for i = 0 to zplane_numpoles - 1 do
-                zplane_poles <- Array.append zplane_poles
-                                             [|blt splane_poles.(i)|]
-              done;
-              for i = 0 to zplane_numzeros - 1 do
-                zplane_zeros <- Array.append zplane_zeros
-                                             [|blt splane_zeros.(i)|]
-              done;
-              while (zplane_numzeros < zplane_numpoles) do
-                zplane_zeros <- Array.append zplane_zeros
-                                             [|{re = -1.0 ; im = 0.}|] ;
-                zplane_numzeros <- zplane_numzeros + 1
-              done;
-            if debug then
-              (
-                Printf.printf "Z-Plane zeros:\n%s\n" (String.concat "\n" (Array.to_list ((Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im) zplane_zeros)))) ;
-                Printf.printf "Z-Plane poles:\n%s\n" (String.concat "\n" (Array.to_list ((Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im) zplane_poles))))
-              )
+                    xv <- Array.make_matrix channels (order + 1) 0. ;
+                    yv <- Array.make_matrix channels (order + 1) 0.
+                | _ -> assert false
+              end ;
+              (* Compute Z-plane zeros & poles using bilinear transform *)
+              self#log#f 4 "S-Plane zeros:" ;
+              self#log#f 4 "%s"
+                (String.concat "\n"
+                   (Array.to_list
+                      (Array.mapi
+                         (fun i a ->
+                            Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im)
+                         splane_zeros))) ;
+              self#log#f 4 "S-Plane poles:" ;
+              self#log#f 4 "%s"
+                (String.concat "\n"
+                   (Array.to_list
+                      (Array.mapi
+                         (fun i a ->
+                            Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im)
+                         splane_poles))) ;
+              zplane_numpoles <- splane_numpoles ;
+              zplane_numzeros <- splane_numzeros ;
+              let blt a =
+                Complex.div (Complex.add {re = 2. ; im = 0.} a)
+                            (Complex.sub {re = 2. ; im = 0.} a)
+              in
+                for i = 0 to zplane_numpoles - 1 do
+                  zplane_poles <- Array.append zplane_poles
+                                    [|blt splane_poles.(i)|]
+                done;
+                for i = 0 to zplane_numzeros - 1 do
+                  zplane_zeros <- Array.append zplane_zeros
+                                    [|blt splane_zeros.(i)|]
+                done;
+                while (zplane_numzeros < zplane_numpoles) do
+                  zplane_zeros <- Array.append zplane_zeros
+                                    [|{re = -1.0 ; im = 0.}|] ;
+                  zplane_numzeros <- zplane_numzeros + 1
+                done;
+                self#log#f 4 "Z-Plane zeros:" ;
+                self#log#f 4 "%s"
+                  (String.concat "\n"
+                     (Array.to_list
+                        (Array.mapi
+                           (fun i a ->
+                              Printf.sprintf "%d: %+.013f %+.013f i."
+                                i a.re a.im)
+                           zplane_zeros))) ;
+                self#log#f 4 "Z-Plane poles:" ;
+                self#log#f 4 "%s"
+                  (String.concat "\n"
+                     (Array.to_list
+                        (Array.mapi
+                           (fun i a ->
+                              Printf.sprintf "%d: %+.013f %+.013f i."
+                                i a.re a.im)
+                           zplane_poles)))
         | Resonator ->
-            (* Compute Z-plane zeros and poles (Resonator). Let's admit we're
-            creating a bandpass filter, we'll transform later if needed. *)
-            if debug then
-              Printf.printf "This is a Resonator filter.\n" ;
+            (* Compute Z-plane zeros and poles (Resonator).
+             * Let's assume we're creating a bandpass filter,
+             * we'll transform later if needed. *)
+            self#log#f 4 "This is a Resonator filter." ;
             zplane_numpoles <- 2 ;
             zplane_numzeros <- 2 ;
             zplane_zeros <- [|cor 1. ; cor (-1.)|] ;
             (* where we want the peak to be *)
             let theta = 2. *. pi *. raw_alpha1 in
-              if (qfactor == infinity) then
-                (
-                  if debug then
-                    Printf.printf "Infinite Q factor!\n" ;
-                  (* oscillator *)
-                  let zp = {re = cos theta ; im = sin theta } in
-                    zplane_poles <- [|zp ; Complex.conj zp|]
-                )
-              else
-                (
-                  (* must iterate to find exact pole positions *)
-                  topcoeffs <- expand zplane_zeros zplane_numzeros ;
-                  let r = exp (cor (-. theta /. (2. *. qfactor)))
-                    and thm = ref theta
-                    and th1 = ref 0.
-                    and th2 = ref pi
-                    and cvg = ref false
-                  in
-                    for i = 0 to 50 do
-                      let zp = r *~ {re = cos !thm ; im = sin !thm} in
-                        zplane_poles <- [|zp ; Complex.conj zp|] ;
-                        botcoeffs <- expand zplane_poles zplane_numpoles ;
-                        let g = evaluate topcoeffs zplane_numzeros
-                                         botcoeffs zplane_numpoles
-                                         {re = cos theta ; im = sin theta}
-                        in
-                          let phi = g.im /. g.re in (* approx to atan2 *)
-                            if (phi > 0.) then
-                              th2 := !thm
-                            else
-                              th1 := !thm ;
-                            if (abs_float phi < 1e-10) then
-                              cvg := true ;
-                            thm := 0.5 *. (!th1 +. !th2)
-                    done ;
-                    (* if we failed to converge ... *)
-                    assert !cvg
-              ) ;
-              xv <- Array.make_matrix (Fmt.channels ()) (zplane_numzeros) 0. ;
-              yv <- Array.make_matrix (Fmt.channels ()) (zplane_numpoles) 0. ;
+              if (qfactor == infinity) then begin
+                self#log#f 4 "Infinite Q factor!" ;
+                (* oscillator *)
+                let zp = {re = cos theta ; im = sin theta } in
+                  zplane_poles <- [|zp ; Complex.conj zp|]
+              end else begin
+                (* must iterate to find exact pole positions *)
+                topcoeffs <- expand zplane_zeros zplane_numzeros ;
+                let r = exp (cor (-. theta /. (2. *. qfactor)))
+                  and thm = ref theta
+                  and th1 = ref 0.
+                  and th2 = ref pi
+                  and cvg = ref false
+                in
+                  for i = 0 to 50 do
+                    let zp = r *~ {re = cos !thm ; im = sin !thm} in
+                      zplane_poles <- [|zp ; Complex.conj zp|] ;
+                      botcoeffs <- expand zplane_poles zplane_numpoles ;
+                      let g = evaluate topcoeffs zplane_numzeros
+                                       botcoeffs zplane_numpoles
+                                       {re = cos theta ; im = sin theta}
+                      in
+                        let phi = g.im /. g.re in (* approx to atan2 *)
+                          if (phi > 0.) then
+                            th2 := !thm
+                          else
+                            th1 := !thm ;
+                          if (abs_float phi < 1e-10) then
+                            cvg := true ;
+                          thm := 0.5 *. (!th1 +. !th2)
+                  done ;
+                  (* if we failed to converge ... *)
+                  assert !cvg
+              end ;
+              xv <- Array.make_matrix channels (zplane_numzeros) 0. ;
+              yv <- Array.make_matrix channels (zplane_numpoles) 0. ;
               (* Do we need to transform to Bandstop or Allpass? *)
-              ( match filter_type with
+              begin match filter_type with
                   | Band_stop ->
                       (* Band-stop filter *)
-                      if debug then
-                        Printf.printf "This is a band-stop filter.\n" ;
+                      self#log#f 4 "This is a band-stop filter." ;
                       (* compute Z-plane pole & zero positions for bandstop
                       resonator (notch filter) *)
                       (* place zeros exactly *)
@@ -321,69 +324,88 @@ object (self)
                         zplane_poles <- [|zp ; Complex.conj zp|]
                   | All_pass ->
                       (* All-pass filter *)
-                      if debug then
-                        Printf.printf "This is an all-pass filter.\n" ;
+                      self#log#f 4 "This is an all-pass filter." ;
                       (* compute Z-plane pole & zero positions for allpass
                       resonator *)
                       zplane_zeros.(0) <- zplane_poles.(0) /~
                         sqrt (cor (Complex.norm zplane_poles.(0))) ;
                       zplane_zeros.(1) <- zplane_poles.(1) /~
-                        sqrt (cor (Complex.norm zplane_poles.(1))) 
+                        sqrt (cor (Complex.norm zplane_poles.(1)))
                   | Band_pass -> () ;
-                  | _ -> assert false ) ) ;
+                  | _ -> assert false
+              end
+      end ;
       (* Now expand the polynomials *)
-      Printf.printf "Expanding polynomials...\n" ;
+      self#log#f 4 "Expanding polynomials..." ;
       topcoeffs <- expand zplane_zeros zplane_numzeros ;
       botcoeffs <- expand zplane_poles zplane_numpoles ;
-      if debug then
-        (
-          Printf.printf "Top coeffs:\n%s\n" (String.concat "\n" (Array.to_list ((Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im) topcoeffs)))) ;
-          Printf.printf "Bottom coeffs:\n%s\n" (String.concat "\n" (Array.to_list ((Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im) botcoeffs))))
-        ) ;
+      self#log#f 4 "Top coeffs:" ;
+      self#log#f 4 "%s"
+        (String.concat "\n"
+           (Array.to_list
+              (Array.mapi
+                 (fun i a ->
+                    Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im)
+                 topcoeffs))) ;
+      self#log#f 4 "Bottom coeffs:" ;
+      self#log#f 4 "%s"
+        (String.concat "\n"
+           (Array.to_list
+              (Array.mapi
+                 (fun i a ->
+                    Printf.sprintf "%d: %+.013f %+.013f i." i a.re a.im)
+                 botcoeffs))) ;
       (* Gain *)
       dc_gain <- evaluate topcoeffs zplane_numzeros botcoeffs
                           zplane_numpoles {re = 1. ; im = 0.} ;
       let theta =
         2. *. pi *. 0.5 *. (raw_alpha1 +. raw_alpha2) (* jwt for centre freq. *)
       in
-        fc_gain <- evaluate topcoeffs zplane_numzeros botcoeffs zplane_numpoles
-                     (Complex.exp {re = 0. ; im = theta}) ;
+      fc_gain <- evaluate topcoeffs zplane_numzeros botcoeffs zplane_numpoles
+                   (Complex.exp {re = 0. ; im = theta}) ;
       hf_gain <- evaluate topcoeffs zplane_numzeros botcoeffs zplane_numpoles
                    {re = -1. ; im = 0.} ;
-      gain <- ( match filter_type with
+      gain <- begin match filter_type with
                 | Band_stop -> Complex.norm (sqrt (dc_gain *~ hf_gain ))
                 | Band_pass
                 | All_pass -> Complex.norm fc_gain
                 | High_pass -> Complex.norm hf_gain
-                | Low_pass -> Complex.norm dc_gain ) ;
-      if debug then
-        Printf.printf "Gains:\nDC:     %+.013f.\nCentre: %+.013f.\nHF:     %+.013f.\nFinal:  %+.013f.\n" (Complex.norm dc_gain) (Complex.norm fc_gain) (Complex.norm hf_gain) gain ;
+                | Low_pass -> Complex.norm dc_gain end ;
+      self#log#f 4 "Gains:" ;
+      self#log#f 4 "DC=%+.013f Centre=%+.013f HF=%+.013f Final=%+.013f."
+        (Complex.norm dc_gain) (Complex.norm fc_gain)
+        (Complex.norm hf_gain) gain ;
       (* X-coeffs *)
       for i = 0 to zplane_numzeros do
         xcoeffs <- Array.append xcoeffs
                      [|topcoeffs.(i).re /. botcoeffs.(zplane_numpoles).re|] ;
       done;
-      if debug then
-        Printf.printf "Xcoeffs:\n%s\n" (String.concat "\n" (Array.to_list ((Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f." i a) xcoeffs)))) ;
+      self#log#f 4 "Xcoeffs:" ;
+      self#log#f 4 "%s"
+        (String.concat "\n"
+           (Array.to_list
+              (Array.mapi
+                 (fun i a -> Printf.sprintf "%d: %+.013f." i a)
+                 xcoeffs))) ;
       (* Y-coeffs *)
       for i = 0 to zplane_numpoles do
         ycoeffs <- Array.append ycoeffs
               [|(0. -. (botcoeffs.(i).re) /. botcoeffs.(zplane_numpoles).re)|] ;
       done;
-      if debug then
-        (
-          Printf.printf "Ycoeffs:\n%s\n" (String.concat "\n" (Array.to_list ((Array.mapi (fun i a -> Printf.sprintf "%d: %+.013f." i a) ycoeffs)))) ;
-          Printf.printf "================== END INIT FILTER %s ==================\n" self#id
-        ) 
-  
+      self#log#f 4 "Ycoeffs:" ;
+      self#log#f 4 "%s"
+        (String.concat "\n"
+           (Array.to_list
+              (Array.mapi
+                 (fun i a -> Printf.sprintf "%d: %+.013f." i a)
+                 ycoeffs))) ;
+      self#log#f 4 "Initialization done."
+
   (* Digital filter based on mkfilter/mkshape/gencode by A.J. Fisher *)
 
   method stype = source#stype
-
   method remaining = source#remaining
-
   method is_ready = source#is_ready
-
   method abort_track = source#abort_track
 
   val mutable v_offs = 0
@@ -391,7 +413,7 @@ object (self)
   method private get_frame buf =
     let offset = AFrame.position buf in
       source#get buf;
-      let b = AFrame.get_float_pcm buf in
+      let b = AFrame.content buf offset in
       let v_len = Array.length xv.(0) in
       let coeffs_len = Array.length xcoeffs in
       let fold_left2 init v coeffs l =
@@ -402,7 +424,7 @@ object (self)
           done;
           !result
       in
-      for c = 0 to (Fmt.channels ()) - 1 do
+      for c = 0 to channels - 1 do
         let xvc = xv.(c) in
         let yvc = yv.(c) in
         let bc = b.(c) in
@@ -420,41 +442,40 @@ object (self)
 end
 
 let () =
+  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.any_fixed in
   Lang.add_operator "filter.iir.butterworth.high"
     [
       "frequency", Lang.float_t, None, Some "Corner frequency" ;
       "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;
-      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
-      "", Lang.source_t, None, None
+      "", Lang.source_t k, None, None
     ]
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.SoundProcessing
     ~descr:"IIR filter"
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
-       let freq, order, debug, src =
+       let freq, order, src =
          Lang.to_float (f "frequency"),
          Lang.to_int (f "order"),
-         Lang.to_bool (f "debug"),
          Lang.to_source (f "") in
-         new iir src Butterworth High_pass order freq 0. 0. debug) ;
+         new iir ~kind src Butterworth High_pass order freq 0. 0.) ;
 
   Lang.add_operator "filter.iir.butterworth.low"
     [
       "frequency", Lang.float_t, None, Some "Corner frequency" ;
       "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;
-      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
-      "", Lang.source_t, None, None
+      "", Lang.source_t k, None, None
     ]
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.SoundProcessing
     ~descr:"IIR filter"
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
-       let freq, order, debug, src =
+       let freq, order, src =
          Lang.to_float (f "frequency"),
          Lang.to_int (f "order"),
-         Lang.to_bool (f "debug"),
          Lang.to_source (f "") in
-         new iir src Butterworth Low_pass order freq 0. 0. debug) ;
+         new iir ~kind src Butterworth Low_pass order freq 0. 0.) ;
 
   Lang.add_operator "filter.iir.butterworth.bandpass"
     [
@@ -462,95 +483,86 @@ let () =
       "frequency2", Lang.float_t, None,
         Some "Second corner frequency" ;
       "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;
-      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
-      "", Lang.source_t, None, None
+      "", Lang.source_t k, None, None
     ]
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.SoundProcessing
     ~descr:"IIR filter"
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
-       let freq1, freq2, order, debug, src =
+       let freq1, freq2, order, src =
          Lang.to_float (f "frequency1"),
          Lang.to_float (f "frequency2"),
          Lang.to_int (f "order"),
-         Lang.to_bool (f "debug"),
          Lang.to_source (f "") in
-         new iir src Butterworth Band_pass order freq1 freq2 0. debug) ;
+         new iir ~kind src Butterworth Band_pass order freq1 freq2 0.) ;
 
   Lang.add_operator "filter.iir.butterworth.bandstop"
     [
       "frequency1", Lang.float_t, None, Some "First corner frequency" ;
-      "frequency2", Lang.float_t, None,
-        Some "Second corner frequency" ;
+      "frequency2", Lang.float_t, None, Some "Second corner frequency" ;
       "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;
-      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
-      "", Lang.source_t, None, None
+      "", Lang.source_t k, None, None
     ]
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.SoundProcessing
     ~descr:"IIR filter"
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
-       let freq1, freq2, order, debug, src =
+       let freq1, freq2, order, src =
          Lang.to_float (f "frequency1"),
          Lang.to_float (f "frequency2"),
          Lang.to_int (f "order"),
-         Lang.to_bool (f "debug"),
          Lang.to_source (f "") in
-         new iir src Butterworth Band_stop order freq1 freq2 0. debug) ;
+         new iir ~kind src Butterworth Band_stop order freq1 freq2 0.) ;
 
   Lang.add_operator "filter.iir.resonator.bandpass"
     [
       "frequency", Lang.float_t, None, Some "Corner frequency" ;
       "q", Lang.float_t, Some (Lang.float 60.), Some "Quality factor" ;
-(*      "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;*)
-      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
-      "", Lang.source_t, None, None
+      "", Lang.source_t k, None, None
     ]
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.SoundProcessing
     ~descr:"IIR filter"
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
-       let freq, q, debug, src =
+       let freq, q, src =
          Lang.to_float (f "frequency"),
          Lang.to_float (f "q"),
-         Lang.to_bool (f "debug"),
          Lang.to_source (f "") in
-         new iir src Resonator Band_pass 0 freq 0. q debug) ;
+         new iir ~kind src Resonator Band_pass 0 freq 0. q) ;
 
   Lang.add_operator "filter.iir.resonator.bandstop"
     [
       "frequency", Lang.float_t, None, Some "Corner frequency" ;
       "q", Lang.float_t, Some (Lang.float 60.), Some "Quality factor" ;
-(*      "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;*)
-      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
-      "", Lang.source_t, None, None
+      "", Lang.source_t k, None, None
     ]
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.SoundProcessing
     ~descr:"IIR filter"
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
-       let freq, q, debug, src =
+       let freq, q, src =
          Lang.to_float (f "frequency"),
          Lang.to_float (f "q"),
-         Lang.to_bool (f "debug"),
          Lang.to_source (f "") in
-         new iir src Resonator Band_pass 0 freq 0. q debug) ;
+         new iir ~kind src Resonator Band_pass 0 freq 0. q) ;
 
   Lang.add_operator "filter.iir.resonator.allpass"
     [
       "frequency", Lang.float_t, None, Some "Corner frequency" ;
       "q", Lang.float_t, Some (Lang.float 60.), Some "Quality factor" ;
-(*      "order", Lang.int_t, Some (Lang.int 4), Some "Filter order" ;*)
-      "debug", Lang.bool_t, Some (Lang.bool false), Some "Debug output" ;
-      "", Lang.source_t, None, None
+      "", Lang.source_t k, None, None
     ]
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.SoundProcessing
     ~descr:"IIR filter"
-    (fun p _ ->
+    (fun p kind ->
        let f v = List.assoc v p in
-       let freq, q, debug, src =
+       let freq, q, src =
          Lang.to_float (f "frequency"),
          Lang.to_float (f "q"),
-         Lang.to_bool (f "debug"),
          Lang.to_source (f "") in
-         new iir src Resonator Band_pass 0 freq 0. q debug)
+         new iir ~kind src Resonator Band_pass 0 freq 0. q)
