@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,17 +22,18 @@
 
 open Source
 
-class map_metadata source rewrite_f insert_missing update strip =
+class map_metadata ~kind source rewrite_f insert_missing update strip =
 object (self)
-  inherit operator [source]
+  inherit operator ~name:"map_metadata" kind [source]
 
   method stype = source#stype
   method is_ready = source#is_ready
   method remaining = source#remaining
   method abort_track = source#abort_track
+  method seek n = source#seek n
 
   method private rewrite m =
-    let m' = Lang.apply rewrite_f ["",Lang.metadata m] in
+    let m' = Lang.apply ~t:Lang.metadata_t rewrite_f ["",Lang.metadata m] in
     let replace_val v =
       let (x,y) = Lang.to_product v in
       let x = Lang.to_string x and y = Lang.to_string y in
@@ -74,6 +75,7 @@ object (self)
 end
 
 let register =
+  let kind = Lang.univ_t 1 in
   Lang.add_operator "map_metadata"
     [ "", Lang.fun_t
             [false,"",
@@ -84,19 +86,20 @@ let register =
       Some "Only update metadata. If false, only returned values \
             will be set as metadata." ;
       "strip", Lang.bool_t, Some (Lang.bool false),
-      Some "Completly remove empty metadata. Operates on both empty values \
+      Some "Completely remove empty metadata. Operates on both empty values \
             and empty metadata chunk.";
       "insert_missing", Lang.bool_t, Some (Lang.bool true),
       Some "Treat track beginnings without metadata as having empty ones. \
             The operational order is: create empty if needed, map and strip \
             if enabled." ;
-      "", Lang.source_t, None, None ]
+      "", Lang.source_t kind, None, None ]
     ~category:Lang.TrackProcessing
     ~descr:"Rewrite metadata on the fly using a function."
-    (fun p _ ->
+    ~kind:(Lang.Unconstrained kind)
+    (fun p kind ->
        let source = Lang.to_source (Lang.assoc "" 2 p) in
        let f = Lang.assoc "" 1 p in
        let update = Lang.to_bool (List.assoc "update" p) in
        let strip = Lang.to_bool (List.assoc "strip" p) in
        let missing = Lang.to_bool (List.assoc "insert_missing" p) in
-         new map_metadata source f missing update strip)
+         new map_metadata ~kind source f missing update strip)
