@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,14 +20,15 @@
 
  *****************************************************************************)
 
-class on_metadata f s =
+class on_metadata ~kind f s =
 object (self)
-  inherit Source.operator [s]
+  inherit Source.operator ~name:"on_metadata" kind [s]
 
   method stype = s#stype
   method is_ready = s#is_ready
   method abort_track = s#abort_track
   method remaining = s#remaining
+  method seek n = s#seek n
 
   method private get_frame ab =
     let p = Frame.position ab in
@@ -36,13 +37,14 @@ object (self)
         (fun (i,m) ->
            if i>=p then begin
              self#log#f 4 "Got metadata at position %d: calling handler..." i ;
-             ignore (Lang.apply f ["",Lang.metadata m])
+             ignore (Lang.apply ~t:Lang.unit_t f ["",Lang.metadata m])
            end)
         (Frame.get_all_metadata ab)
 
 end
 
 let () =
+  let kind = Lang.univ_t 1 in
   Lang.add_operator "on_metadata"
     [ "",
       Lang.fun_t
@@ -51,10 +53,11 @@ let () =
       None,
       Some ("Function called on every metadata packet in the stream."^
             " It should be fast because it is ran in the main thread.") ;
-      "", Lang.source_t, None, None ]
+      "", Lang.source_t kind, None, None ]
     ~category:Lang.TrackProcessing
     ~descr:"Call a given handler on metadata packets."
-    (fun p _ ->
+    ~kind:(Lang.Unconstrained kind)
+    (fun p kind ->
        let f = Lang.assoc "" 1 p in
        let s = Lang.to_source (Lang.assoc "" 2 p) in
-         new on_metadata f s)
+         new on_metadata ~kind f s)
