@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -42,16 +42,14 @@ type child = {
 }
 
 (** The switch can either happen at any time in the stream (insensitive)
-  * or only at track limits (sensitive), in which case it is possible
-  * to have the transition being triggered a while before end of track --
-  * anticipation parameter specified as a number of ticks. *)
-type track_mode = Sensitive of int | Insensitive
+  * or only at track limits (sensitive). *)
+type track_mode = Sensitive | Insensitive
 
-class virtual
-  switch ?(mode=Sensitive 0) ?(replay_meta=true) (cases : child list) =
+class virtual switch ~kind ~name
+  ?(mode=Sensitive) ?(replay_meta=true) (cases : child list) =
 object (self)
 
-  inherit operator (List.map (fun x -> x.source) cases) as super
+  inherit operator ~name kind (List.map (fun x -> x.source) cases) as super
 
   val mutable selected : (child*source) option = None
 
@@ -147,30 +145,35 @@ object (self)
                   (if forget then " forgetful" else "") ;
                 old_s#leave (self:>source) ;
                 to_finish <- old_s::to_finish ;
-                let old_source =
-                  if forget then Blank.empty else old_c.source
-                in
-                let new_source =
-                  (* Force insertion of old metadata if relevant.
-                   * It can't be done in a static way: we need to start
-                   * pulling data to see if new metadata comes out, in case
-                   * the source was shared and kept streaming from somewhere
-                   * else (this is thanks to Frame.get_chunk).
-                   * A quicker hack might have been doable if there wasn't a
-                   * transition in between. *)
-                  match c.cur_meta with
-                    | Some m when replay_meta ->
-                        new Insert_metadata.replay m c.source
-                    | _ -> c.source
-                in
-                let s =
-                  Lang.to_source
-                    (Lang.apply c.transition
-                       [ "",Lang.source old_source ;
-                         "",Lang.source new_source ])
-                in
-                  s#get_ready activation ;
-                  selected <- Some (c,s)
+                Clock.collect_after begin fun () ->
+                  let old_source =
+                    if forget then Blank.empty kind else old_c.source
+                  in
+                  let new_source =
+                    (* Force insertion of old metadata if relevant.
+                     * It can't be done in a static way: we need to start
+                     * pulling data to see if new metadata comes out, in case
+                     * the source was shared and kept streaming from somewhere
+                     * else (this is thanks to Frame.get_chunk).
+                     * A quicker hack might have been doable if there wasn't a
+                     * transition in between. *)
+                    match c.cur_meta with
+                      | Some m when replay_meta ->
+                          new Insert_metadata.replay ~kind m c.source
+                      | _ -> c.source
+                  in
+                  let s =
+                    let t = Lang.source_t (Lang.kind_type_of_frame_kind kind) in
+                    Lang.to_source
+                      (Lang.apply ~t
+                         c.transition
+                         [ "",Lang.source old_source ;
+                           "",Lang.source new_source ])
+                  in
+                    Clock.unify s#clock self#clock ;
+                    s#get_ready activation ;
+                    selected <- Some (c,s)
+                end
             | _ ->
                 (* We are staying on the same child,
                  * don't start a new track. *)
@@ -220,11 +223,7 @@ object (self)
           if Frame.is_partial ab then
             reselect ~forget:true ()
           else
-            if
-              match mode with
-                | Insensitive -> true
-                | Sensitive d -> s#remaining > 0 && s#remaining <= d
-            then
+            if mode = Insensitive then
               reselect ()
 
   method remaining =
@@ -241,7 +240,7 @@ end
 
 (** Common tools for Lang bindings of switch operators *)
 
-let common = [
+let common kind = [
   "track_sensitive", Lang.bool_t, Some (Lang.bool true),
   Some "Re-select only on end of tracks." ;
 
@@ -249,29 +248,31 @@ let common = [
   Some "Replay the last metadata of a child when switching to it \
         in the middle of a track." ;
 
-  "before", Lang.float_t, Some (Lang.float 0.),
-  Some "EXPERIMENTAL: for track_sensitive switches, \
-        trigger transitions before the end of track." ;
-
+  let transition_t =
+    Lang.fun_t
+      [false,"",Lang.source_t kind;
+       false,"",Lang.source_t kind]
+      (Lang.source_t kind)
+  in
   "transitions",
-  Lang.list_t
-    (Lang.fun_t [ false,"",Lang.source_t ;
-                  false,"",Lang.source_t ] Lang.source_t),
-  Some (Lang.list []),
+  Lang.list_t transition_t,
+  Some (Lang.list transition_t []),
   Some "Transition functions, \
         padded with <code>fun (x,y) -> y</code> functions."
 ]
 
-let default_transition =
-  Lang.val_fun [ "","x",None ; "","y",None ] (fun e _ -> List.assoc "y" e)
+let default_transition k =
+  let t = Lang.source_t (Lang.kind_type_of_frame_kind k) in
+    Lang.val_fun
+      [ "","x",t,None ; "","y",t,None ] ~ret_t:t
+      (fun e _ -> List.assoc "y" e)
 
-let extract_common p l =
+let extract_common ~kind p l =
   let ts =
-    let before = Lang.to_float (List.assoc "before" p) in
-      if Lang.to_bool (List.assoc "track_sensitive" p) then
-        Sensitive (Fmt.ticks_of_seconds before)
-      else
-        Insensitive
+    if Lang.to_bool (List.assoc "track_sensitive" p) then
+      Sensitive
+    else
+      Insensitive
   in
   let tr =
     let tr = Lang.to_list (List.assoc "transitions" p) in
@@ -280,7 +281,7 @@ let extract_common p l =
                                ((List.assoc "transitions" p),
                                 "Too many transitions")) ;
       if ltr < l then
-        tr @ (Utils.make_list (l-ltr) default_transition)
+        tr @ (Utils.make_list (l-ltr) (default_transition kind))
       else
         tr
   in
@@ -289,7 +290,7 @@ let extract_common p l =
 
 (** Switch: switch according to user-defined predicates. *)
 
-let satisfied f = Lang.to_bool (Lang.apply f [])
+let satisfied f = Lang.to_bool (Lang.apply ~t:Lang.bool_t f [])
 let trivially_true = function
   | { Lang.value =
         Lang.Fun (_,_,_,{ Lang_values.term = Lang_values.Bool true }) } -> true
@@ -297,11 +298,12 @@ let trivially_true = function
 
 let third (_,_,s) = s
 
-class
-  lang_switch mode ?replay_meta (children : (Lang.value * bool * child) list) =
+class lang_switch ~kind
+  mode ?replay_meta (children : (Lang.value * bool * child) list) =
 object (self)
 
-  inherit switch ~mode ?replay_meta (List.map third children)
+  inherit
+    switch ~name:"switch" ~kind ~mode ?replay_meta (List.map third children)
 
   method private select =
     let selected s =
@@ -333,20 +335,22 @@ object (self)
 end
 
 let () =
+  let kind = Lang.univ_t 1 in
   let pred_t = Lang.fun_t [] Lang.bool_t in
-  let proto = 
-    [ "single", Lang.list_t Lang.bool_t, Some (Lang.list []),
+  let proto =
+    [ "single", Lang.list_t Lang.bool_t, Some (Lang.list Lang.bool_t []),
       Some "Forbid the selection of a branch for two tracks in a row. \
             The empty list stands for <code>[false,...,false]</code>." ;
-      "", Lang.list_t (Lang.product_t pred_t Lang.source_t), None,
+      "", Lang.list_t (Lang.product_t pred_t (Lang.source_t kind)), None,
       Some "Sources with the predicate telling when they can be played." ]
   in
     Lang.add_operator "switch"
       ~category:Lang.TrackProcessing
-      ~descr:("At the beginning of a track, select the first source "^
-              "whose predicate is true.")
-      (common@proto)
-      (fun p _ ->
+      ~descr:"At the beginning of a track, select the first source \
+              whose predicate is true."
+      (common kind @ proto)
+      ~kind:(Lang.Unconstrained kind)
+      (fun p kind ->
          let children =
            List.map
              (fun p ->
@@ -354,7 +358,9 @@ let () =
                   pred, Lang.to_source s)
            (Lang.to_list (List.assoc "" p))
          in
-         let replay_meta,ts,tr = extract_common p (List.length children) in
+         let replay_meta,ts,tr =
+           extract_common ~kind p (List.length children)
+         in
          let singles =
            List.map Lang.to_bool (Lang.to_list (List.assoc "single" p))
          in
@@ -377,13 +383,13 @@ let () =
                           (List.assoc "single" p,
                            "there should be exactly one flag per children"))
          in
-           new lang_switch ~replay_meta ts children)
+           new lang_switch ~kind ~replay_meta ts children)
 
 (** Fallback selector: switch to the first ready source. *)
-class fallback ?replay_meta mode children =
+class fallback ~kind ?replay_meta mode children =
 object
 
-  inherit switch ~mode ?replay_meta children
+  inherit switch ~name:"fallback" ~kind ~mode ?replay_meta children
 
   method private select =
     try Some (List.find (fun s -> s.source#is_ready) children) with
@@ -398,30 +404,35 @@ object
 end
 
 let () =
+  let kind = Lang.univ_t 1 in
   let proto =
     [ "",
-      Lang.list_t Lang.source_t,
+      Lang.list_t (Lang.source_t kind),
       None,
       Some "Select the first ready source in this list." ]
   in
     Lang.add_operator "fallback" ~category:Lang.TrackProcessing
       ~descr:"At the beginning of each track, select the first ready child."
-      (common@proto)
-      (fun p _ ->
+      (common kind @ proto)
+      ~kind:(Lang.Unconstrained kind)
+      (fun p kind ->
          let children = Lang.to_source_list (List.assoc "" p) in
-         let replay_meta,ts,tr = extract_common p (List.length children) in
+         let replay_meta,ts,tr =
+           extract_common ~kind p (List.length children)
+         in
          let children =
            List.map2
              (fun t s -> { transition = t ; cur_meta = None ; source = s })
              tr children
          in
-           new fallback ~replay_meta ts children)
+           new fallback ~kind ~replay_meta ts children)
 
 (** Random switch *)
 exception Found of child
-class random ?replay_meta strict mode children =
+class random ~kind ?replay_meta strict mode children =
+  let name = if strict then "quota" else "random" in
 object
-  inherit switch ?replay_meta ~mode (List.map snd children)
+  inherit switch ~name ~kind ?replay_meta ~mode (List.map snd children)
 
   val mutable pos = -1
 
@@ -455,14 +466,18 @@ end
 
 let () =
   let add name strict descr weight_descr =
+    let kind = Lang.univ_t 1 in
     Lang.add_operator name ~descr ~category:Lang.TrackProcessing
-      (common @
-       [ "weights", Lang.list_t Lang.int_t, Some (Lang.list []),
+      (common  kind @
+       [ "weights", Lang.list_t Lang.int_t, Some (Lang.list Lang.int_t []),
          Some weight_descr ;
-         "", Lang.list_t Lang.source_t, None, None ])
-      (fun p _ ->
+         "", Lang.list_t (Lang.source_t kind), None, None ])
+      ~kind:(Lang.Unconstrained kind)
+      (fun p kind ->
          let children = Lang.to_source_list (List.assoc "" p) in
-         let replay_meta,ts,tr = extract_common p (List.length children) in
+         let replay_meta,ts,tr =
+           extract_common ~kind p (List.length children)
+         in
          let weights =
            List.map Lang.to_int (Lang.to_list (List.assoc "weights" p))
          in
@@ -483,7 +498,7 @@ let () =
                 (fun tr s -> { transition = tr ; source = s ; cur_meta = None })
                 tr children)
          in
-           new random ~replay_meta strict ts children)
+           new random ~kind ~replay_meta strict ts children)
   in
     add "random" false
       "At the beginning of every track, select a random ready child."
