@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2010 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,33 +25,48 @@
 open Alsa
 open Dtools
 
-(** ALSA should be quiet *)
-let () = no_stderr_report ()
-
-class output dev start source =
-  let buffer_length = Fmt.samples_per_frame () in
-  let buffer_chans = Fmt.channels () in
-  let blank () = Array.init buffer_chans (fun _ -> Array.make buffer_length 0.) in
+class output ~kind ~clock_safe ~infallible
+             ~on_stop ~on_start ~start dev source =
+  let buffer_length = AFrame.size () in
+  let buffer_chans = (Frame.type_of_kind kind).Frame.audio in
+  let alsa_buffer = Alsa_settings.alsa_buffer#get in
+  let blank () =
+    Array.init buffer_chans (fun _ -> Array.make buffer_length 0.)
+  in
   let nb_blocks = Alsa_settings.conf_buffer_length#get in
-  let samples_per_second = Fmt.samples_per_second () in
+  let samples_per_second = Lazy.force Frame.audio_rate in
   let periods = Alsa_settings.periods#get in
-  (* Force these parameters for now. *)
-  let infallible = true in
-  let on_stop () = () in
-  let on_start () = () in
+  let name = Printf.sprintf "alsa_out(%s)" dev in
 object (self)
-  inherit Output.output
-              ~infallible ~on_stop ~on_start
-              ~name:"output.alsa" ~kind:"output.alsa" source start
-  inherit [float array array] IoRing.output ~nb_blocks ~blank
-                                 ~blocking:true () as ioring
+  inherit
+    Output.output
+      ~infallible ~on_stop ~on_start ~content_kind:kind
+      ~name ~output_kind:"output.alsa" source start
+    as super
+  inherit [float array array] IoRing.output ~nb_blocks ~blank as ioring
+
+  method private set_clock =
+    super#set_clock ;
+    if clock_safe then
+      Clock.unify self#clock
+        (Clock.create_known ((Alsa_settings.get_clock ()):>Clock.clock))
+
+  method output_start =
+    ioring#output_start ;
+    if clock_safe then
+      (Alsa_settings.get_clock ())#register_blocking_source
+
+  method output_stop =
+    ioring#output_stop ;
+    if clock_safe then
+      (Alsa_settings.get_clock ())#unregister_blocking_source
 
   val mutable device = None
 
   val mutable alsa_rate = samples_per_second
   val samplerate_converter = Audio_converter.Samplerate.create buffer_chans
   val mutable alsa_write =
-    (fun pcm buf ofs len -> Pcm.writen_float pcm buf ofs len) 
+    (fun pcm buf ofs len -> Pcm.writen_float pcm buf ofs len)
 
   method get_device =
     match device with
@@ -60,7 +75,7 @@ object (self)
           self#log#f 3 "Using ALSA %s." (Alsa.get_version ()) ;
           let dev = Pcm.open_pcm dev [Pcm.Playback] [] in
           let params = Pcm.get_params dev in
-          let bufsize =
+          let bufsize,periods =
              (
                try
                  Pcm.set_access dev params Pcm.Access_rw_noninterleaved ;
@@ -74,7 +89,7 @@ object (self)
                     alsa_write <-
                       (fun pcm buf ofs len ->
                          let sbuf = String.create (2 * len * Array.length buf) in
-                         let _ =  Float_pcm.to_s16le buf ofs len sbuf 0 in
+                         Audio.S16LE.of_audio buf ofs sbuf 0 len;
                          Pcm.writei pcm sbuf 0 len
                       )
              );
@@ -84,8 +99,16 @@ object (self)
               * This setting is critical as a too small bufsize will easily result in
               * underruns when the thread isn't fast enough.
               * TODO make it customizable *)
-             Pcm.set_periods dev params periods Dir_eq;
-             Pcm.set_buffer_size_near dev params 65536
+             let bufsize = 
+               if alsa_buffer > 0 then
+                 Pcm.set_buffer_size_near dev params alsa_buffer 
+               else
+                 Pcm.get_buffer_size_max params
+             in
+             if periods > 0 then
+               Pcm.set_periods dev params periods Dir_eq;
+             bufsize,
+             (fst (Pcm.get_periods_max params))
           in
           self#log#f 3 "Samplefreq=%dHz, Bufsize=%dB, Frame=%dB, Periods=%d"
             alsa_rate bufsize (Pcm.get_frame_size params) periods ;
@@ -104,10 +127,10 @@ object (self)
     let dev = self#get_device in
     try
       let len = Array.length data.(0) in
-      let rec f pos = 
+      let rec f pos =
         if pos < len then
           let ret = alsa_write dev data pos (len - pos) in
-          f (pos+ret) 
+          f (pos+ret)
       in
       f 0
     with
@@ -126,19 +149,19 @@ object (self)
         else raise e
 
   method output_send buf =
-    let buf = AFrame.get_float_pcm buf in
+    let buf = AFrame.content buf 0 in
     let ratio = float alsa_rate /. float samples_per_second in
-    let buf = Audio_converter.Samplerate.resample samplerate_converter 
-                  ratio buf 0 (Array.length buf.(0)) 
+    let buf = Audio_converter.Samplerate.resample samplerate_converter
+                  ratio buf 0 (Array.length buf.(0))
     in
     let f data =
-      for c = 0 to Array.length buf - 1 do
-        Float_pcm.float_blit buf.(c) 0 data.(c) 0 (Array.length buf.(0))
-      done
+      Audio.blit buf 0 data 0 (Audio.duration buf)
     in
     ioring#put_block f
 
-  method output_reset = ()
+  method output_reset = 
+    self#close ;
+    ignore(self#get_device)
 end
 
 (* It is registered in Alsa_io. *)
