@@ -1,6 +1,7 @@
 (*****************************************************************************
 
-  Copyright 2003-2010 Savonet team
+  Liquidsoap, a programmable audio stream generator.
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,65 +24,79 @@
 
 let bytes_per_sample = 2
 
-class output
-       ~infallible ~on_stop ~on_start 
+class output ~kind ~clock_safe
+       ~infallible ~on_stop ~on_start
        ~nb_blocks ~server source =
-  let channels = Fmt.channels () in
-  let samples_per_frame = Fmt.samples_per_frame () in
-  let samples_per_second = Fmt.samples_per_second () in
+  let channels = (Frame.type_of_kind kind).Frame.audio in
+  let samples_per_frame = AFrame.size () in
+  let seconds_per_frame = Frame.seconds_of_audio samples_per_frame in
+  let samples_per_second = Lazy.force Frame.audio_rate in
   let blank () =
     String.make (samples_per_frame * channels * bytes_per_sample) '0'
   in
 object (self)
-  inherit Output.output 
-              ~infallible ~on_stop ~on_start
-              ~name:"output.jack" ~kind:"output.jack" source true
-  inherit [string] IoRing.output ~nb_blocks ~blank 
-                                 ~blocking:true () as ioring
+  inherit Output.output
+            ~infallible ~on_stop ~on_start ~content_kind:kind
+            ~name:"output.jack" ~output_kind:"output.jack" source true
+          as super
+  inherit [string] IoRing.output ~nb_blocks ~blank as ioring
+
+  method private set_clock =
+    super#set_clock ;
+    if clock_safe then
+      Clock.unify self#clock
+        (Clock.create_known ((Bjack_in.bjack_clock ()):>Clock.clock))
+
+  method output_start =
+    ioring#output_start ;
+    if clock_safe then (Bjack_in.bjack_clock ())#register_blocking_source
+
+  method output_stop =
+    ioring#output_stop ;
+    if clock_safe then (Bjack_in.bjack_clock ())#unregister_blocking_source
 
   val mutable device = None
 
-  method get_device = 
+  method get_device =
     match device with
-      | None -> 
+      | None ->
           (* Wait for things to settle *)
-          Thread.delay (5. *. (Fmt.seconds_per_frame ()));
+          Thread.delay (5. *. seconds_per_frame);
           let server_name =
             match server with "" -> None | s -> Some s
           in
-	  let dev = 
-            Bjack.open_t 
-	      ~rate:samples_per_second ~bits_per_sample:(bytes_per_sample * 8)
+          let dev =
+            Bjack.open_t
+              ~rate:samples_per_second ~bits_per_sample:(bytes_per_sample * 8)
               ~input_channels:0 ~output_channels:channels ~flags:[] ?server_name
-              ~ringbuffer_size:(nb_blocks*samples_per_frame*bytes_per_sample) 
+              ~ringbuffer_size:(nb_blocks*samples_per_frame*bytes_per_sample)
               ~client_name:self#id () in
-          Bjack.set_all_volume dev 75 ;
+          Bjack.set_all_volume dev 100 ;
           device <- Some dev ;
           dev
       | Some d -> d
 
-  method push_block data = 
+  method push_block data =
     let dev = self#get_device in
     let len = String.length data in
     let remaining = ref (len - (Bjack.write dev data)) in
     while !remaining > 0 do
-      Thread.delay (Fmt.seconds_per_frame () /. 2.) ;
+      Thread.delay (seconds_per_frame /. 2.) ;
       let tmp = Str.string_after data (len - !remaining) in
       let written = Bjack.write dev tmp in
       remaining := !remaining - written
     done
 
-  method close = 
+  method close =
     match device with
-      | Some d -> 
+      | Some d ->
           Bjack.close d ;
           device <- None
       | None -> ()
 
   method output_send wav =
     let push data =
-      ignore (Float_pcm.to_s16le (AFrame.get_float_pcm wav) 0 (AFrame.size wav)
-                data 0)
+      Audio.S16LE.of_audio (AFrame.content wav 0) 0 data 0 samples_per_frame
     in
     ioring#put_block push
 
@@ -89,30 +104,36 @@ object (self)
 end
 
 let () =
-  Lang.add_operator "output.jack"
+  let k = Lang.kind_type_of_kind_format ~fresh:1 Lang.audio_any in
+  Lang.add_operator "output.jack" ~active:true
    ( Output.proto @
-    [ "buffer_size",
-      Lang.int_t, Some (Lang.int 2),
-      Some "Set buffer size, in frames.";
-     "server",
-      Lang.string_t, Some (Lang.string ""),
-      Some "Jack server to connect to.";
-      "", Lang.source_t, None, None ])
+    [ "clock_safe",
+        Lang.bool_t, Some (Lang.bool true),
+        Some "Force the use of the dedicated bjack clock." ;
+      "buffer_size",
+        Lang.int_t, Some (Lang.int 2),
+        Some "Set buffer size, in frames.";
+      "server",
+        Lang.string_t, Some (Lang.string ""),
+        Some "Jack server to connect to.";
+      "", Lang.source_t k, None, None ])
+    ~kind:(Lang.Unconstrained k)
     ~category:Lang.Output
     ~descr:"Output stream to jack."
-    (fun p _ ->
+    (fun p kind ->
        let source = List.assoc "" p in
+       let clock_safe = Lang.to_bool (List.assoc "clock_safe" p) in
        let nb_blocks = Lang.to_int (List.assoc "buffer_size" p) in
        let server = Lang.to_string (List.assoc "server" p) in
        let infallible = not (Lang.to_bool (List.assoc "fallible" p)) in
        let on_start =
          let f = List.assoc "on_start" p in
-           fun () -> ignore (Lang.apply f [])
+           fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
        in
        let on_stop =
          let f = List.assoc "on_stop" p in
-           fun () -> ignore (Lang.apply f [])
+           fun () -> ignore (Lang.apply ~t:Lang.unit_t f [])
        in
-         ((new output 
+         ((new output ~kind ~clock_safe
                   ~infallible ~on_start ~on_stop
                   ~nb_blocks ~server source):>Source.source))
