@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -27,8 +27,8 @@ open Dtools
 let error_translator =
   function
     | Cry.Error _ as e ->
-       raise (Utils.Translation (Cry.string_of_error e))
-    | _ -> ()
+       Some (Cry.string_of_error e)
+    | _ -> None
 
 let () = Utils.register_error_translator error_translator
 
@@ -76,6 +76,12 @@ struct
               samplerate = Some m.Encoder.MP3.samplerate ;
               channels = Some (if m.Encoder.MP3.stereo then 2 else 1)
             }
+        | Encoder.Shine m ->
+            { quality = None ;
+              bitrate = Some m.Encoder.Shine.bitrate ;
+              samplerate = Some m.Encoder.Shine.samplerate ;
+              channels = Some m.Encoder.Shine.channels
+            }
         | Encoder.AACPlus m ->
             { quality = None ;
               bitrate = Some m.Encoder.AACPlus.bitrate ;
@@ -88,12 +94,24 @@ struct
               samplerate = Some m.Encoder.VoAacEnc.samplerate ;
               channels = Some m.Encoder.VoAacEnc.channels
             }
+        | Encoder.FdkAacEnc m ->
+            { quality = None ;
+              bitrate = Some m.Encoder.FdkAacEnc.bitrate ;
+              samplerate = Some m.Encoder.FdkAacEnc.samplerate ;
+              channels = Some m.Encoder.FdkAacEnc.channels
+            }
         | Encoder.External m ->
             { quality = None ;
               bitrate = None ;
               samplerate = Some m.Encoder.External.samplerate ;
               channels = Some m.Encoder.External.channels
             }
+        | Encoder.GStreamer m ->
+            { quality    = None ;
+              bitrate    = None ;
+              samplerate = None ;
+              channels   = Some (Encoder.GStreamer.audio_channels m)
+            }
         | Encoder.Flac m ->
             { quality = Some (string_of_int m.Encoder.Flac.compression) ;
               bitrate = None ;
@@ -472,7 +490,7 @@ object (self)
         Cry.connect connection source ;
         self#log#f 3 "Connection setup was successful." ;
         let c = Cry.get_connection_data connection in
-        let () = Liq_sockets.set_tcp_nodelay c.Cry.data_socket true in
+        Unix.setsockopt c.Cry.data_socket Unix.TCP_NODELAY true;
         (* Execute on_connect hook. *)
         on_connect () ;
       with
