@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -147,7 +147,7 @@ object (self)
          (Unix.gettimeofday () > reload_delay +. open_date &&
           (Lang.to_bool (Lang.apply ~t:Lang.bool_t reload_predicate []))) then
           begin
-            self#log#f 3 "Re-opening output file..." ;
+            self#log#f 3 "Re-opening output pipe..." ;
             (* #output_stop can trigger #send,
              * the [reopening] flag avoids loops *)
             reopening <- true ;
@@ -173,7 +173,7 @@ end
   * it. *)
 
 let chan_proto kind arg_doc =
-  ["flush", Lang.int_t, Some (Lang.bool false),
+  ["flush", Lang.bool_t, Some (Lang.bool false),
    Some "Perform a flush after each write."] @
   (pipe_proto kind arg_doc)
 
