@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -40,15 +40,32 @@ let conf_mime_types =
       "If you feel that new mime-types should be permanently added, please";
       "contact the developpers."
     ]
+let conf_cue_in_metadata =
+  Conf.string ~p:(conf_playlists#plug "cue_in_metadata") ~d:"liq_cue_in"
+    "Cue in metadata for playlists with track index."
+    ~comments:["Some playlists format, such as CUE files specify index points to start";
+      "tracks playback. In this case, tracks are resolved to a annotate: request with";
+      "a cue-in metadata containing the index. If you want to make use of this index,";
+      "you should specify here what label you want for this metadata and use the cue_cut";
+      "operator on the resulting source"]
+let conf_cue_out_metadata =
+  Conf.string ~p:(conf_playlists#plug "cue_out_metadata") ~d:"liq_cue_out"
+    "Cue out metadata for playlists with track index."
+    ~comments:["Some playlists format, such as CUE files specify index points to start";
+      "tracks playback. In this case, tracks are resolved to a annotate: request with";
+      "a cue-in metadata containing the index. If you want to make use of this index,";
+      "you should specify here what label you want for this metadata and use the cue_cut";
+      "operator on the resulting source"]
 
 (** A playlist is list of metadatas,uri *)
 type playlist = ((string * string) list * string) list
 
 (** A plugin is a boolean and a parsing function *)
-type plugin = { 
+type plugin = {
   strict : bool; (* true is the format can be detected *)
-  parser: string -> playlist (* The parser is expected to respect the order
-                                of the files in the playlist. *)
+  parser: ?pwd:string -> string -> playlist
+    (* The parser is expected to respect the order
+       of the files in the playlist. *)
 }
 
 (** Parsers are given a string and return a list of metadatas,uri, if possible. *)
@@ -56,6 +73,15 @@ let parsers : plugin Plug.plug =
   Plug.create
     ~doc:"Method to parse playlist." "playlist formats"
 
+let get_file ?pwd file =
+  match pwd with
+    | Some pwd ->
+      if Http.is_url pwd && not (Http.is_url file) then
+        pwd ^ file
+      else
+        let f = Filename.concat pwd file in
+        if Sys.file_exists f then f else file
+    | None -> file
 
 exception Exit of (string*playlist)
 (** Get a valid parser for [string].
@@ -64,16 +90,16 @@ exception Exit of (string*playlist)
   * Being based on file extension is weak, and troublesome when accessing a
   * remote file -- that would force us to create a local temporary file with the
   * same extension. *)
-let search_valid string =
+let search_valid ?pwd string =
   try
     let plugins = parsers#get_all in
     (* Try strict plugins first *)
     let compare (_,a) (_,b) = compare b.strict a.strict in
     let plugins = List.sort compare plugins in
-    List.iter 
+    List.iter
       (fun (format,plugin) ->
            log#f 4 "Trying %s parser" format ;
-	   match try Some (plugin.parser string) with _ -> None
+	   match try Some (plugin.parser ?pwd string) with _ -> None
 	      with
 	        | Some d -> raise (Exit (format,d))
 		| None -> () )
@@ -82,4 +108,4 @@ let search_valid string =
     raise Not_found
   with
     | Exit (format,d) -> format,d
-	    
+
