@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -71,7 +71,8 @@ let connect () =
   let write s =
     let len = String.length s in
       if debug then Printf.printf "W: %s%!" s;
-      assert (Unix.send socket s 0 len [] = len)
+      let l = Unix.send socket s 0 len [] in
+      assert (l = len)
   in
     Unix.connect socket sockaddr;
     socket, read, write
