@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2012 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@ let conf =
   Conf.void ~p:(Configure.conf#plug "request") "requests configuration"
 let grace_time =
   Conf.float ~p:(conf#plug "grace_time") ~d:600.
-    "Time after which a destroyed request cannot be accessed anymore."
+    "Time (in seconds) after which a destroyed request cannot be accessed anymore."
 
 let log = Log.make ["request"]
 
@@ -175,6 +175,28 @@ let indicator ?(metadata=Hashtbl.create 10) ?temporary s = {
   metadata = metadata
 }
 
+(** Length *)
+let dresolvers_doc =
+  "Methods to extract duration from a file."
+let dresolvers =
+  Plug.create ~doc:dresolvers_doc ~insensitive:true
+    "audio file formats (duration)"
+
+exception Duration of float
+let duration file =
+  try
+    dresolvers#iter
+      (fun name resolver ->
+        try
+          let ans = resolver file in
+          raise (Duration ans)
+        with
+        | Duration e -> raise (Duration e)
+        | _ -> ());
+    raise Not_found
+  with
+  | Duration d -> d
+
 (** Manage requests' metadata *)
 
 let toplevel_metadata t =
@@ -285,6 +307,13 @@ let conf_override_metadata =
       "Allow metadata resolvers to override metadata already \
        set through annotate: or playlist resolution for instance."
 
+let conf_duration =
+  Conf.bool ~p:(conf_metadata_decoders#plug "duration") ~d:false
+    "Compute duration in the \"duration\" metadata, if the metadata is not \
+     already present. This can take a long time and the use of this option is \
+     not recommended: the proper way is to have a script precompute the \
+     \"duration\" metadata."
+
 (** Sys.file_exists doesn't make a difference between existing files
   * and files without enough permissions to list their attributes,
   * for example when they are in a directory without x permission.
@@ -327,6 +356,13 @@ let local_check t =
                              Hashtbl.replace indicator.metadata
                               k (cleanup v))
                          ans;
+                     if conf_duration#get && get_metadata t "duration" = None then
+                       (
+                         try
+                           Hashtbl.replace indicator.metadata "duration" (string_of_float (duration name))
+                         with
+                         | Not_found -> ()
+                       )
                    with
                      | _ -> ()) (get_decoders conf_metadata_decoders
                                      mresolvers) ;
@@ -365,29 +401,6 @@ let get_filename t =
   else
     None
 
-(** Length *)
-let dresolvers_doc =
-  "Methods to extract duration from a file."
-let dresolvers =
-  Plug.create ~doc:dresolvers_doc ~insensitive:true
-    "audio file formats (duration)"
-
-exception Duration of float
-let duration file =
-  try
-    dresolvers#iter
-      (fun name resolver ->
-        try
-         let ans = resolver file in
-         raise (Duration ans)
-        with 
-          | Duration e -> raise (Duration e) 
-          | _ -> () ) ; 
-      raise Not_found
-  with
-    | Duration d -> d
-
-
 let update_metadata t =
   let replace = Hashtbl.replace t.root_metadata in
     replace "rid" (string_of_int t.id) ;
@@ -584,7 +597,8 @@ let resolve t timeout =
                   pop_indicator t
             end
         | None ->
-            log#f 3 "Nonexistent file or ill-formed URI %S!" i.string ;
+            let log_level = if i.string = "" then 4 else 3 in
+            log#f log_level "Nonexistent file or ill-formed URI %S!" i.string ;
             add_log t "Nonexistent file or ill-formed URI!" ;
             pop_indicator t
   in
