@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -319,6 +319,13 @@ object (self)
   val mutable dynamic_activations : operator list list = []
   val mutable static_activations  : operator list list = []
 
+  (* List of callbacks executed when source shuts down. *)
+  val mutable on_shutdown = []
+  val on_shutdown_m = Mutex.create()
+  method on_shutdown fn =
+    (Tutils.mutexify on_shutdown_m (fun () ->
+      on_shutdown <- fn :: on_shutdown)) ()
+
   (* contains: (ns,descr,usage,name,f) *)
   val mutable commands = []
   val mutable ns_kind = "unknown"
@@ -404,6 +411,12 @@ object (self)
       self#update_caching_mode ;
       if static_activations = [] && dynamic_activations = [] then begin
         source_log#f 4 "Source %s gets down." id ;
+        (Tutils.mutexify on_shutdown_m
+           (fun () ->
+             List.iter (fun fn -> try fn() with _ -> ()) on_shutdown;
+             on_shutdown <- []
+           )
+        ) ();
         self#sleep ;
         List.iter
           (fun (_,_,name,_) ->
@@ -477,7 +490,10 @@ object (self)
          *   having to worry about availability.
          * So we add this branch, which makes the whole protocol a bit
          * sloppy because it removes any constraint tying #is_ready and
-         * #get. *)
+         * #get. It prevents the detection of "bad" calls of #get without
+         * having checked #is_ready. It also makes it really important
+         * to have #is_ready = true during tracks, otherwise this bit
+         * of code will forcefully end the track! *)
         Frame.add_break buf (Frame.position buf)
       else
       let b = Frame.breaks buf in
