@@ -1,5 +1,7 @@
 (* Some structured exceptions *)
 
+include Stdlib
+
 type error = Socket | Response | UrlDecoding
 exception Error of error
 
@@ -12,8 +14,8 @@ let string_of_error e =
 (** Error translator *)
 let error_translator e =
    match e with
-     | Error e -> raise (Utils.Translation (string_of_error e))
-     | _ -> ()
+     | Error e -> Some (string_of_error e)
+     | _ -> None
 
 let () = Utils.register_error_translator error_translator
 
@@ -114,7 +116,9 @@ let http_sanitize url =
     let sub = Pcre.exec ~rex:basic_rex url in
     let host,path = Pcre.get_substring sub 1,Pcre.get_substring sub 2 in
     let encode path =
-      let path = Pcre.split ~pat:"/" path in
+      (* Pcre.split removes empty strings and thus removes trailing '/' which
+         can change the semantics of the URL... *)
+      let path = String.split_char '/' path in
       (* We decode the path, in case it was already encoded. *)
       let path =
         List.map (fun x -> url_encode ~plus:false (url_decode x)) path
@@ -144,6 +148,14 @@ let http_sanitize url =
   with
     | _ -> url
 
+let is_url path =
+  Pcre.pmatch ~pat:"^http://.+" path
+
+let dirname url =
+  let rex = Pcre.regexp "^(http://.+/)[^/]*$" in
+  let s = Pcre.exec ~rex url in
+  Pcre.get_substring s 1
+
 (** HTTP functions. *)
 
 let connect ?bind_address host port =
@@ -174,7 +186,7 @@ let disconnect socket =
     | _ -> ()
 
 let read ?(log=fun _ -> ()) ~timeout socket buflen =
-  Utils.wait_for ~log `Read socket timeout;
+  Tutils.wait_for ~log `Read socket timeout;
   match buflen with
     | Some buflen ->
         let buf = String.create buflen in
@@ -213,7 +225,7 @@ let read_crlf ?(log=fun _ -> ()) ?(max=4096) ~timeout socket =
       (* This is quite ridiculous but we have 
        * no way to know how much data is available
        * in the socket.. *)
-      Utils.wait_for ~log `Read socket timeout;
+      Tutils.wait_for ~log `Read socket timeout;
       let h = Unix.read socket c 0 1 in
         if h < 1 then
           loop := false
@@ -232,7 +244,7 @@ let read_crlf ?(log=fun _ -> ()) ?(max=4096) ~timeout socket =
 let request ?(log=fun _ -> ()) ~timeout socket request =
   if
     let len = String.length request in
-      Utils.wait_for ~log `Write socket timeout;
+      Tutils.wait_for ~log `Write socket timeout;
       Unix.write socket request 0 len < len
   then
     raise Socket ;
