@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -85,9 +85,12 @@ struct
   let format_of_encoder =
     function
       | Encoder.MP3 _ -> Some mpeg
-      | Encoder.AACPlus _ -> Some aacplus
+      | Encoder.Shine _ -> Some mpeg
+      | Encoder.AACPlus _ -> Some aac
       | Encoder.VoAacEnc _ -> Some aac
+      | Encoder.FdkAacEnc _ -> Some aac
       | Encoder.External _ -> None
+      | Encoder.GStreamer _ -> None
       | Encoder.Flac _ -> Some flac
       | Encoder.WAV _ -> Some wav
       | Encoder.Ogg _ -> Some ogg
