@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,14 +20,59 @@
 
  *****************************************************************************)
 
-let get_some = function Some x -> x | None -> assert false
+let get_some = function
+  | Some x -> x
+  | None -> assert false
+
+let maybe f = function
+  | Some x -> Some (f x)
+  | None   -> None
+
+let maydo f = function
+  | Some x -> f x
+  | None -> ()
+
+let some_or none = function
+  | Some x -> x
+  | None   -> none
 
 (* Force locale to C *)
 external force_locale : unit -> unit = "liquidsoap_set_locale"
 
+(** General configuration *)
+let conf = Dtools.Conf.void "Liquidsoap configuration"
+
 let () = 
   force_locale ()
 
+(* Resolve a path. *)
+let resolve_path ?cwd path =
+  let cwd = 
+    match cwd with
+      | None     -> Sys.getcwd()
+      | Some cwd -> cwd
+  in
+  if Filename.is_relative path then
+   begin
+     Filename.concat cwd path
+   end
+  else
+    path
+
+(* Getenv with default value. *)
+let getenv ~default value =
+  try
+    Sys.getenv value
+  with
+    | Not_found -> default
+
+(* Getenv with option result. *)
+let getenv_opt value =
+  try
+    Some (Sys.getenv value)
+  with
+    | Not_found -> None
+
 (* Several list utilities *)
 
 let rec make_list n v = if n = 0 then [] else v::(make_list (n-1) v)
@@ -51,6 +96,65 @@ let hashtbl_get : ('a,'b) Hashtbl.t -> 'a -> 'b option =
   fun h k ->
     try Some (Hashtbl.find h k) with Not_found -> None
 
+(** Unescapt a string. *)
+let unescape s =
+  try
+    Scanf.sscanf s "%S" (fun u -> u)
+  with
+    | _ -> s
+
+(* Unescape a given char in a string, i.e. \\c -> c *)
+let unescape_char c s =
+  let len = String.length s in
+  let rec f ~escaped cur pos =
+    if pos >= len then
+      (if escaped then Printf.sprintf "%s\\" cur else cur)
+    else
+      let c   = s.[pos] in
+      let pos = pos + 1 in
+      match c with
+        | '\\' when not escaped ->
+            f ~escaped:true cur pos
+        | x    when x = c ->
+            f ~escaped:false (Printf.sprintf "%s%c" cur c) pos
+        | x    when escaped -> 
+            f ~escaped:false (Printf.sprintf "%s\\%c" cur x) pos
+        | x ->
+            f ~escaped:false (Printf.sprintf "%s%c" cur x) pos
+  in
+  f ~escaped:false "" 0
+
+(** Parse strings of the form foo <sep> bar,
+ *  trying to be clever on escaping <sep> in foo
+ *  and bar.. *)
+let split ~sep s = 
+  let rec split cur s =
+    let len = String.length s in
+    let rec get_index escaped pos =
+      if pos == len then
+        pos
+      else
+        match s.[pos] with
+          | '\\' when not escaped -> get_index true  (pos+1)
+          | x    when x == sep && not escaped -> pos
+          | _                     -> get_index false (pos+1) 
+    in
+    let pos = get_index false 0 in
+    if pos == len then
+      s :: cur
+     else
+       if pos == len -1 then
+         String.sub s 0 pos :: cur
+       else
+         split (String.sub s 0 pos :: cur ) (String.sub s (pos+1) (len-pos-1))
+  in
+  List.map (unescape_char sep) (List.rev (split [] s))
+
+(** Remove trailing and leading spaces. *)
+let trim s =
+  Pcre.replace ~rex:(Pcre.regexp "\\s+$")
+    (Pcre.replace ~rex:(Pcre.regexp "^\\s+") s)
+
 (** Remove the first element satisfying a predicate, raising Not_found
   * if none is found. *)
 let remove_one f l =
@@ -78,29 +182,6 @@ let really_read fd buf ofs len =
     done;
     !l
 
-(* There seems to be an issue under win32 where 
- * some sockets are left in non-blocking mode
- * after Unix.select. See: http://caml.inria.fr/mantis/view.php?id=5328
- * 
- * Since we do not use non-blocking mode at all
- * in liquidsoap, this wrapper ensures that all socket are set back to
- * blocking mode after a call to select under win32.. *)
-let select r w e t =
-  let ret = Unix.select r w e t in
-  if Sys.os_type <> "Win32" then
-    ret
-  else
-   begin
-    let f x =
-       try
-        Unix.clear_nonblock x
-       with _ -> ()
-    in
-    let f = List.iter f in
-    f r; f w; f e;
-    ret
-   end
-
 (* Read all data from a given filename.
  * We cannot use really_input with the 
  * reported length of the file because
@@ -126,36 +207,7 @@ let read_all filename =
   close_in channel ;
   Buffer.contents contents
 
-exception Timeout
-
-(* Wait for [`Read], [`Write] or [`Both] for
- * at most [timeout] seconds on the
- * given [socket]. Raises [Timeout] 
- * if timeout is reached. *)
-let wait_for ?(log=fun _ -> ()) event socket timeout = 
-  let max_time = Unix.gettimeofday () +. timeout in
-  let r, w = 
-    match event with
-      | `Read -> [socket],[]
-      | `Write -> [],[socket]
-      | `Both -> [socket],[socket]
-  in
-  let rec wait t =
-    let l,l',_ = select r w [] t in
-    if l=[] && l'=[] then begin
-      log (Printf.sprintf "No network activity for %.02f second(s)." t);
-      let current_time = Unix.gettimeofday () in
-      if current_time >= max_time then
-       begin
-        log "Network activity timeout! Disconnecting source." ;
-        raise Timeout 
-       end
-      else
-        wait (min 1. (max_time -. current_time))
-    end
-  in wait (min 1. timeout)
-
-(* Drop all but then [len] last bytes. *)
+(* Drop the first [len] bytes. *)
 let buffer_drop buffer len =
   let size = Buffer.length buffer in
   assert (len <= size) ;
@@ -169,9 +221,6 @@ let buffer_drop buffer len =
  * and override it with Printexc's implementation
  * if present.. *)
 
-(* Exception translation *)
-exception Translation of string
-
 let error_translators = Queue.create ()
 
 let register_error_translator x = Queue.push x error_translators
@@ -179,19 +228,16 @@ let register_error_translator x = Queue.push x error_translators
 let unix_translator = 
   function
     | Unix.Unix_error (code,name,param) ->
-      raise (Translation 
-          (Printf.sprintf "%s in %s(%s)" (Unix.error_message code) 
-                                         name param))
-    | _ -> ()
+       Some (Printf.sprintf "%s in %s(%s)" (Unix.error_message code) 
+                                           name param)
+    | _ -> None
 
 let () = register_error_translator unix_translator
 
 let exception_printer e =
- try
-   Queue.iter (fun f -> f e) error_translators ;
-   None
- with
-   | Translation x -> Some x
+  Queue.fold
+    (fun cur f -> if cur <> None then cur else f e)
+    None error_translators
 
 let register_printer _ = 
     raise Not_found
@@ -347,6 +393,7 @@ let interpolate =
   let var    = "\\$(\\([^()$]+\\))" in
   let re_if  =
     (* Groups              1           2 (3 4)   5       6 (7 8)      *)
+    (* TODO: use unescape above? *)
     Str.regexp ("\\$(if +"^var^" *, *"^quoted^"\\( *, *"^quoted^"\\)?)")
   in
   let unescape = Str.global_replace (Str.regexp "\\\\\\(.\\)") "\\1" in
@@ -423,9 +470,7 @@ let rec mkdir ~perm dir =
 
 (** Expand ~ notation in filenames. *)
 let home_unrelate =
-  let home =
-    try Some (Sys.getenv "HOME") with Not_found -> None
-  in
+  let home = getenv_opt "HOME" in
   let unrel s =
     let len = String.length s in
       if len < 2 then
@@ -451,9 +496,9 @@ let home_unrelate =
 
 let get_tempdir () =
   if Sys.os_type = "Win32" then
-    (try Sys.getenv "TEMP" with Not_found -> "C:\temp")
+    getenv ~default:"C:\\temp" "TEMP"
   else
-    (try Sys.getenv "TMPDIR" with Not_found -> "/tmp")
+    getenv ~default:"/tmp" "TMPDIR"
 
 (** Decode Base64-encoded data *)
 let decode64 s =
@@ -547,3 +592,42 @@ let name_of_sockaddr ?(rev_dns=true) ?(show_port=false) a =
 let uptime =
   let base = Unix.gettimeofday () in
     fun () -> Unix.gettimeofday () -. base
+
+(** Generate a string which can be used as a parameter name. *)
+let normalize_parameter_string s =
+  let s =
+    Pcre.substitute
+      ~pat:"( *\\([^\\)]*\\)| *\\[[^\\]]*\\])"
+      ~subst:(fun _ -> "") s
+  in
+  let s = Pcre.substitute ~pat:"(\\.+|\\++)" ~subst:(fun _ -> "") s in
+  let s = Pcre.substitute ~pat:" +$" ~subst:(fun _ -> "") s in
+  let s = Pcre.substitute ~pat:"( +|/+|-+)" ~subst:(fun _ -> "_") s in
+  let s = Pcre.substitute ~pat:"\"" ~subst:(fun _ -> "") s in
+  let s = String.lowercase s in
+  (* Identifiers cannot begin with a digit. *)
+  let s = if Pcre.pmatch ~pat:"^[0-9]" s then "_"^s else s in
+  s
+
+(** A function to reopen a file descriptor
+  * Thanks to Xavier Leroy!
+  * Ref: http://caml.inria.fr/pub/ml-archives/caml-list/2000/01/
+  *      a7e3bbdfaab33603320d75dbdcd40c37.en.html
+  *)
+let reopen_out outchan filename =
+  flush outchan;
+  let fd1 = Unix.descr_of_out_channel outchan in
+  let fd2 =
+    Unix.openfile filename [Unix.O_WRONLY] 0o666
+  in
+  Unix.dup2 fd2 fd1;
+  Unix.close fd2
+
+(** The same for inchan *)
+let reopen_in inchan filename =
+  let fd1 = Unix.descr_of_in_channel inchan in
+  let fd2 =
+    Unix.openfile filename [Unix.O_RDONLY] 0o666
+  in
+  Unix.dup2 fd2 fd1;
+  Unix.close fd2
