@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2012 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -51,9 +51,8 @@ exception Not_a_wav_file of string
 let error_translator =
   function
     | Not_a_wav_file x ->
-       raise (Utils.Translation
-         (Printf.sprintf "Wave error: %s" x))
-    | _ -> ()
+       Some (Printf.sprintf "Wave error: %s" x)
+    | _ -> None
 
 let () = Utils.register_error_translator error_translator
 
@@ -94,7 +93,7 @@ let read_header read_ops ic =
       raise (Not_a_wav_file "Bad header: \"fmt \" expected");
 
     let fmt_len = read_int ic in
-    if fmt_len <> 0x10 then
+    if fmt_len < 0x10 then
       raise (Not_a_wav_file "Bad header: invalid \"fmt \" length");
     if read_short ic <> 1 then
       raise (Not_a_wav_file "Bad header: unhandled codec");
@@ -104,6 +103,8 @@ let read_header read_ops ic =
     let byt_per_sec = read_int ic in
     let byt_per_samp= read_short ic in
     let bit_per_samp= read_short ic in
+    (* The fmt header can be padded *)
+    if fmt_len > 0x10 then ignore (read_int_num_bytes ic (fmt_len - 0x10));
 
     let header = ref (read_string ic 4) in
       (* Skip unhandled chunks. *)
@@ -165,6 +166,9 @@ let info w =
 let channels w = w.channels_number
 let sample_rate w = w.sample_rate
 let sample_size w = w.bits_per_sample
+let data_length w = w.length_of_data_to_follow
+(** Length of the data in samples. *)
+let data_samples w = w.length_of_data_to_follow / ((sample_size w / 8) * channels w)
 
 let close w =
   w.read_ops.close w.ic
