@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2011 Savonet team
+  Copyright 2003-2013 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ open Source
 
 class midimeter ~kind source =
 object
-  inherit operator kind [source] as super
+  inherit operator ~name:"midimeter" kind [source] as super
 
   method stype = source#stype
   method is_ready = source#is_ready
